from __future__ import annotations

import pendulum


locale = "da"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "for 1 sekund siden"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "for 2 sekunder siden"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "for 1 minut siden"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "for 2 minutter siden"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "for 1 time siden"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "for 2 timer siden"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "for 1 dag siden"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "for 2 dage siden"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "for 1 uge siden"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "for 2 uger siden"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "for 1 måned siden"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "for 2 måneder siden"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "for 1 år siden"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "for 2 år siden"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "om 1 sekund"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "1 sekund efter"
    assert d2.diff_for_humans(d, locale=locale) == "1 sekund før"

    assert d.diff_for_humans(d2, True, locale=locale) == "1 sekund"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "2 sekunder"



from __future__ import annotations

import pytest

import pendulum

from pendulum import Date
from tests.conftest import assert_date


def test_start_of_day():
    d = Date.today()
    new = d.start_of("day")
    assert isinstance(new, Date)
    assert_date(new, d.year, d.month, d.day)


def test_end_of_day():
    d = Date.today()
    new = d.end_of("day")
    assert isinstance(new, Date)
    assert_date(new, d.year, d.month, d.day)


def test_start_of_week():
    d = Date(2016, 10, 20)
    new = d.start_of("week")
    assert isinstance(new, Date)
    assert_date(new, d.year, d.month, 17)


def test_end_of_week():
    d = Date(2016, 10, 20)
    new = d.end_of("week")
    assert isinstance(new, Date)
    assert_date(new, d.year, d.month, 23)


def test_start_of_month_is_fluid():
    d = Date.today()
    assert isinstance(d.start_of("month"), Date)


def test_start_of_month_from_now():
    d = Date.today()
    new = d.start_of("month")
    assert_date(new, d.year, d.month, 1)


def test_start_of_month_from_last_day():
    d = Date(2000, 1, 31)
    new = d.start_of("month")
    assert_date(new, 2000, 1, 1)


def test_start_of_year_is_fluid():
    d = Date.today()
    new = d.start_of("year")
    assert isinstance(new, Date)


def test_start_of_year_from_now():
    d = Date.today()
    new = d.start_of("year")
    assert_date(new, d.year, 1, 1)


def test_start_of_year_from_first_day():
    d = Date(2000, 1, 1)
    new = d.start_of("year")
    assert_date(new, 2000, 1, 1)


def test_start_of_year_from_last_day():
    d = Date(2000, 12, 31)
    new = d.start_of("year")
    assert_date(new, 2000, 1, 1)


def test_end_of_month_is_fluid():
    d = Date.today()
    assert isinstance(d.end_of("month"), Date)


def test_end_of_month_from_now():
    d = Date.today().start_of("month")
    new = d.start_of("month")
    assert_date(new, d.year, d.month, 1)


def test_end_of_month():
    d = Date(2000, 1, 1).end_of("month")
    new = d.end_of("month")
    assert_date(new, 2000, 1, 31)


def test_end_of_month_from_last_day():
    d = Date(2000, 1, 31)
    new = d.end_of("month")
    assert_date(new, 2000, 1, 31)


def test_end_of_year_is_fluid():
    d = Date.today()
    assert isinstance(d.end_of("year"), Date)


def test_end_of_year_from_now():
    d = Date.today().end_of("year")
    new = d.end_of("year")
    assert_date(new, d.year, 12, 31)


def test_end_of_year_from_first_day():
    d = Date(2000, 1, 1)
    new = d.end_of("year")
    assert_date(new, 2000, 12, 31)


def test_end_of_year_from_last_day():
    d = Date(2000, 12, 31)
    new = d.end_of("year")
    assert_date(new, 2000, 12, 31)


def test_start_of_decade_is_fluid():
    d = Date.today()
    assert isinstance(d.start_of("decade"), Date)


def test_start_of_decade_from_now():
    d = Date.today()
    new = d.start_of("decade")
    assert_date(new, d.year - d.year % 10, 1, 1)


def test_start_of_decade_from_first_day():
    d = Date(2000, 1, 1)
    new = d.start_of("decade")
    assert_date(new, 2000, 1, 1)


def test_start_of_decade_from_last_day():
    d = Date(2009, 12, 31)
    new = d.start_of("decade")
    assert_date(new, 2000, 1, 1)


def test_end_of_decade_is_fluid():
    d = Date.today()
    assert isinstance(d.end_of("decade"), Date)


def test_end_of_decade_from_now():
    d = Date.today()
    new = d.end_of("decade")
    assert_date(new, d.year - d.year % 10 + 9, 12, 31)


def test_end_of_decade_from_first_day():
    d = Date(2000, 1, 1)
    new = d.end_of("decade")
    assert_date(new, 2009, 12, 31)


def test_end_of_decade_from_last_day():
    d = Date(2009, 12, 31)
    new = d.end_of("decade")
    assert_date(new, 2009, 12, 31)


def test_start_of_century_is_fluid():
    d = Date.today()
    assert isinstance(d.start_of("century"), Date)


def test_start_of_century_from_now():
    d = Date.today()
    new = d.start_of("century")
    assert_date(new, d.year - d.year % 100 + 1, 1, 1)


def test_start_of_century_from_first_day():
    d = Date(2001, 1, 1)
    new = d.start_of("century")
    assert_date(new, 2001, 1, 1)


def test_start_of_century_from_last_day():
    d = Date(2100, 12, 31)
    new = d.start_of("century")
    assert_date(new, 2001, 1, 1)


def test_end_of_century_is_fluid():
    d = Date.today()
    assert isinstance(d.end_of("century"), Date)


def test_end_of_century_from_now():
    now = Date.today()
    d = now.end_of("century")
    assert_date(d, now.year - now.year % 100 + 100, 12, 31)


def test_end_of_century_from_first_day():
    d = Date(2001, 1, 1)
    new = d.end_of("century")
    assert_date(new, 2100, 12, 31)


def test_end_of_century_from_last_day():
    d = Date(2100, 12, 31)
    new = d.end_of("century")
    assert_date(new, 2100, 12, 31)


def test_average_is_fluid():
    d = Date.today().average()
    assert isinstance(d, Date)


def test_average_from_same():
    d1 = pendulum.date(2000, 1, 31)
    d2 = pendulum.date(2000, 1, 31).average(d1)
    assert_date(d2, 2000, 1, 31)


def test_average_from_greater():
    d1 = pendulum.date(2000, 1, 1)
    d2 = pendulum.date(2009, 12, 31).average(d1)
    assert_date(d2, 2004, 12, 31)


def test_average_from_lower():
    d1 = pendulum.date(2009, 12, 31)
    d2 = pendulum.date(2000, 1, 1).average(d1)
    assert_date(d2, 2004, 12, 31)


def test_start_of():
    d = pendulum.date(2013, 3, 31)

    with pytest.raises(ValueError):
        d.start_of("invalid")


def test_end_of_invalid_unit():
    d = pendulum.date(2013, 3, 31)

    with pytest.raises(ValueError):
        d.end_of("invalid")



from __future__ import annotations

import pickle

from copy import deepcopy
from datetime import timedelta

import pendulum

from tests.conftest import assert_duration


def test_pickle() -> None:
    it = pendulum.duration(days=3, seconds=2456, microseconds=123456)
    s = pickle.dumps(it)
    it2 = pickle.loads(s)

    assert it == it2


def test_comparison_to_timedelta() -> None:
    duration = pendulum.duration(days=3)

    assert duration < timedelta(days=4)


def test_deepcopy() -> None:
    duration = pendulum.duration(months=1)
    copied_duration = deepcopy(duration)

    assert copied_duration == duration
    assert_duration(copied_duration, months=1)



from __future__ import annotations

import calendar
import datetime
import traceback

from typing import TYPE_CHECKING
from typing import Any
from typing import Callable
from typing import ClassVar
from typing import Optional
from typing import cast
from typing import overload

import pendulum

from pendulum.constants import ATOM
from pendulum.constants import COOKIE
from pendulum.constants import MINUTES_PER_HOUR
from pendulum.constants import MONTHS_PER_YEAR
from pendulum.constants import RFC822
from pendulum.constants import RFC850
from pendulum.constants import RFC1036
from pendulum.constants import RFC1123
from pendulum.constants import RFC2822
from pendulum.constants import RSS
from pendulum.constants import SECONDS_PER_DAY
from pendulum.constants import SECONDS_PER_MINUTE
from pendulum.constants import W3C
from pendulum.constants import YEARS_PER_CENTURY
from pendulum.constants import YEARS_PER_DECADE
from pendulum.date import Date
from pendulum.day import WeekDay
from pendulum.exceptions import PendulumException
from pendulum.helpers import add_duration
from pendulum.interval import Interval
from pendulum.time import Time
from pendulum.tz import UTC
from pendulum.tz import local_timezone
from pendulum.tz.timezone import FixedTimezone
from pendulum.tz.timezone import Timezone


if TYPE_CHECKING:
    from typing_extensions import Literal
    from typing_extensions import Self
    from typing_extensions import SupportsIndex


class DateTime(datetime.datetime, Date):
    EPOCH: ClassVar[DateTime]
    min: ClassVar[DateTime]
    max: ClassVar[DateTime]

    # Formats

    _FORMATS: ClassVar[dict[str, str | Callable[[datetime.datetime], str]]] = {
        "atom": ATOM,
        "cookie": COOKIE,
        "iso8601": lambda dt: dt.isoformat("T"),
        "rfc822": RFC822,
        "rfc850": RFC850,
        "rfc1036": RFC1036,
        "rfc1123": RFC1123,
        "rfc2822": RFC2822,
        "rfc3339": lambda dt: dt.isoformat("T"),
        "rss": RSS,
        "w3c": W3C,
    }

    _MODIFIERS_VALID_UNITS: ClassVar[list[str]] = [
        "second",
        "minute",
        "hour",
        "day",
        "week",
        "month",
        "year",
        "decade",
        "century",
    ]

    _EPOCH: datetime.datetime = datetime.datetime(1970, 1, 1, tzinfo=UTC)

    @classmethod
    def create(
        cls,
        year: SupportsIndex,
        month: SupportsIndex,
        day: SupportsIndex,
        hour: SupportsIndex = 0,
        minute: SupportsIndex = 0,
        second: SupportsIndex = 0,
        microsecond: SupportsIndex = 0,
        tz: str | float | Timezone | FixedTimezone | None | datetime.tzinfo = UTC,
        fold: int = 1,
        raise_on_unknown_times: bool = False,
    ) -> Self:
        """
        Creates a new DateTime instance from a specific date and time.
        """
        if tz is not None:
            tz = pendulum._safe_timezone(tz)

        dt = datetime.datetime(
            year, month, day, hour, minute, second, microsecond, fold=fold
        )

        if tz is not None:
            dt = tz.convert(dt, raise_on_unknown_times=raise_on_unknown_times)

        return cls(
            dt.year,
            dt.month,
            dt.day,
            dt.hour,
            dt.minute,
            dt.second,
            dt.microsecond,
            tzinfo=dt.tzinfo,
            fold=dt.fold,
        )

    @classmethod
    def instance(
        cls,
        dt: datetime.datetime,
        tz: str | Timezone | FixedTimezone | datetime.tzinfo | None = UTC,
    ) -> Self:
        tz = dt.tzinfo or tz

        if tz is not None:
            tz = pendulum._safe_timezone(tz, dt=dt)

        return cls.create(
            dt.year,
            dt.month,
            dt.day,
            dt.hour,
            dt.minute,
            dt.second,
            dt.microsecond,
            tz=tz,
            fold=dt.fold,
        )

    @overload
    @classmethod
    def now(cls, tz: datetime.tzinfo | None = None) -> Self: ...

    @overload
    @classmethod
    def now(cls, tz: str | Timezone | FixedTimezone | None = None) -> Self: ...

    @classmethod
    def now(
        cls, tz: str | Timezone | FixedTimezone | datetime.tzinfo | None = None
    ) -> Self:
        """
        Get a DateTime instance for the current date and time.
        """
        if tz is None or tz == "local":
            dt = datetime.datetime.now(local_timezone())
        elif tz is UTC or tz == "UTC":
            dt = datetime.datetime.now(UTC)
        else:
            dt = datetime.datetime.now(UTC)
            tz = pendulum._safe_timezone(tz)
            dt = dt.astimezone(tz)

        return cls(
            dt.year,
            dt.month,
            dt.day,
            dt.hour,
            dt.minute,
            dt.second,
            dt.microsecond,
            tzinfo=dt.tzinfo,
            fold=dt.fold,
        )

    @classmethod
    def utcnow(cls) -> Self:
        """
        Get a DateTime instance for the current date and time in UTC.
        """
        return cls.now(UTC)

    @classmethod
    def today(cls) -> Self:
        return cls.now()

    @classmethod
    def strptime(cls, time: str, fmt: str) -> Self:
        return cls.instance(datetime.datetime.strptime(time, fmt))

    # Getters/Setters

    def set(
        self,
        year: int | None = None,
        month: int | None = None,
        day: int | None = None,
        hour: int | None = None,
        minute: int | None = None,
        second: int | None = None,
        microsecond: int | None = None,
        tz: str | float | Timezone | FixedTimezone | datetime.tzinfo | None = None,
    ) -> Self:
        if year is None:
            year = self.year
        if month is None:
            month = self.month
        if day is None:
            day = self.day
        if hour is None:
            hour = self.hour
        if minute is None:
            minute = self.minute
        if second is None:
            second = self.second
        if microsecond is None:
            microsecond = self.microsecond
        if tz is None:
            tz = self.tz

        return self.__class__.create(
            year, month, day, hour, minute, second, microsecond, tz=tz, fold=self.fold
        )

    @property
    def float_timestamp(self) -> float:
        return self.timestamp()

    @property
    def int_timestamp(self) -> int:
        # Workaround needed to avoid inaccuracy
        # for far into the future datetimes
        dt = datetime.datetime(
            self.year,
            self.month,
            self.day,
            self.hour,
            self.minute,
            self.second,
            self.microsecond,
            tzinfo=self.tzinfo,
            fold=self.fold,
        )

        delta = dt - self._EPOCH

        return delta.days * SECONDS_PER_DAY + delta.seconds

    @property
    def offset(self) -> int | None:
        return self.get_offset()

    @property
    def offset_hours(self) -> float | None:
        offset = self.get_offset()

        if offset is None:
            return None

        return offset / SECONDS_PER_MINUTE / MINUTES_PER_HOUR

    @property
    def timezone(self) -> Timezone | FixedTimezone | None:
        if not isinstance(self.tzinfo, (Timezone, FixedTimezone)):
            return None

        return self.tzinfo

    @property
    def tz(self) -> Timezone | FixedTimezone | None:
        return self.timezone

    @property
    def timezone_name(self) -> str | None:
        tz = self.timezone

        if tz is None:
            return None

        return tz.name

    @property
    def age(self) -> int:
        return self.date().diff(self.now(self.tz).date(), abs=False).in_years()

    def is_local(self) -> bool:
        return self.offset == self.in_timezone(pendulum.local_timezone()).offset

    def is_utc(self) -> bool:
        return self.offset == 0

    def is_dst(self) -> bool:
        return self.dst() != datetime.timedelta()

    def get_offset(self) -> int | None:
        utcoffset = self.utcoffset()
        if utcoffset is None:
            return None

        return int(utcoffset.total_seconds())

    def date(self) -> Date:
        return Date(self.year, self.month, self.day)

    def time(self) -> Time:
        return Time(self.hour, self.minute, self.second, self.microsecond)

    def naive(self) -> Self:
        """
        Return the DateTime without timezone information.
        """
        return self.__class__(
            self.year,
            self.month,
            self.day,
            self.hour,
            self.minute,
            self.second,
            self.microsecond,
        )

    def on(self, year: int, month: int, day: int) -> Self:
        """
        Returns a new instance with the current date set to a different date.
        """
        return self.set(year=int(year), month=int(month), day=int(day))

    def at(
        self, hour: int, minute: int = 0, second: int = 0, microsecond: int = 0
    ) -> Self:
        """
        Returns a new instance with the current time to a different time.
        """
        return self.set(
            hour=hour, minute=minute, second=second, microsecond=microsecond
        )

    def in_timezone(self, tz: str | Timezone | FixedTimezone) -> Self:
        """
        Set the instance's timezone from a string or object.
        """
        tz = pendulum._safe_timezone(tz)

        dt = self
        if not self.timezone:
            dt = dt.replace(fold=1)

        return tz.convert(dt)

    def in_tz(self, tz: str | Timezone | FixedTimezone) -> Self:
        """
        Set the instance's timezone from a string or object.
        """
        return self.in_timezone(tz)

    # STRING FORMATTING

    def to_time_string(self) -> str:
        """
        Format the instance as time.
        """
        return self.format("HH:mm:ss")

    def to_datetime_string(self) -> str:
        """
        Format the instance as date and time.
        """
        return self.format("YYYY-MM-DD HH:mm:ss")

    def to_day_datetime_string(self) -> str:
        """
        Format the instance as day, date and time (in english).
        """
        return self.format("ddd, MMM D, YYYY h:mm A", locale="en")

    def to_atom_string(self) -> str:
        """
        Format the instance as ATOM.
        """
        return self._to_string("atom")

    def to_cookie_string(self) -> str:
        """
        Format the instance as COOKIE.
        """
        return self._to_string("cookie", locale="en")

    def to_iso8601_string(self) -> str:
        """
        Format the instance as ISO 8601.
        """
        string = self._to_string("iso8601")

        if self.tz and self.tz.name == "UTC":
            string = string.replace("+00:00", "Z")

        return string

    def to_rfc822_string(self) -> str:
        """
        Format the instance as RFC 822.
        """
        return self._to_string("rfc822")

    def to_rfc850_string(self) -> str:
        """
        Format the instance as RFC 850.
        """
        return self._to_string("rfc850")

    def to_rfc1036_string(self) -> str:
        """
        Format the instance as RFC 1036.
        """
        return self._to_string("rfc1036")

    def to_rfc1123_string(self) -> str:
        """
        Format the instance as RFC 1123.
        """
        return self._to_string("rfc1123")

    def to_rfc2822_string(self) -> str:
        """
        Format the instance as RFC 2822.
        """
        return self._to_string("rfc2822")

    def to_rfc3339_string(self) -> str:
        """
        Format the instance as RFC 3339.
        """
        return self._to_string("rfc3339")

    def to_rss_string(self) -> str:
        """
        Format the instance as RSS.
        """
        return self._to_string("rss")

    def to_w3c_string(self) -> str:
        """
        Format the instance as W3C.
        """
        return self._to_string("w3c")

    def _to_string(self, fmt: str, locale: str | None = None) -> str:
        """
        Format the instance to a common string format.
        """
        if fmt not in self._FORMATS:
            raise ValueError(f"Format [{fmt}] is not supported")

        fmt_value = self._FORMATS[fmt]
        if callable(fmt_value):
            return fmt_value(self)

        return self.format(fmt_value, locale=locale)

    def __str__(self) -> str:
        return self.isoformat(" ")

    def __repr__(self) -> str:
        us = ""
        if self.microsecond:
            us = f", {self.microsecond}"

        repr_ = "{klass}({year}, {month}, {day}, {hour}, {minute}, {second}{us}"

        if self.tzinfo is not None:
            repr_ += ", tzinfo={tzinfo}"

        repr_ += ")"

        return repr_.format(
            klass=self.__class__.__name__,
            year=self.year,
            month=self.month,
            day=self.day,
            hour=self.hour,
            minute=self.minute,
            second=self.second,
            us=us,
            tzinfo=repr(self.tzinfo),
        )

    # Comparisons
    def closest(self, *dts: datetime.datetime) -> Self:  # type: ignore[override]
        """
        Get the closest date to the instance.
        """
        pdts = [self.instance(x) for x in dts]

        return min((abs(self - dt), dt) for dt in pdts)[1]

    def farthest(self, *dts: datetime.datetime) -> Self:  # type: ignore[override]
        """
        Get the farthest date from the instance.
        """
        pdts = [self.instance(x) for x in dts]

        return max((abs(self - dt), dt) for dt in pdts)[1]

    def is_future(self) -> bool:
        """
        Determines if the instance is in the future, ie. greater than now.
        """
        return self > self.now(self.timezone)

    def is_past(self) -> bool:
        """
        Determines if the instance is in the past, ie. less than now.
        """
        return self < self.now(self.timezone)

    def is_long_year(self) -> bool:
        """
        Determines if the instance is a long year

        See link `https://en.wikipedia.org/wiki/ISO_8601#Week_dates`_
        """
        return (
            DateTime.create(self.year, 12, 28, 0, 0, 0, tz=self.tz).isocalendar()[1]
            == 53
        )

    def is_same_day(self, dt: datetime.datetime) -> bool:  # type: ignore[override]
        """
        Checks if the passed in date is the same day
        as the instance current day.
        """
        dt = self.instance(dt)

        return self.to_date_string() == dt.to_date_string()

    def is_anniversary(  # type: ignore[override]
        self, dt: datetime.datetime | None = None
    ) -> bool:
        """
        Check if its the anniversary.
        Compares the date/month values of the two dates.
        """
        if dt is None:
            dt = self.now(self.tz)

        instance = self.instance(dt)

        return (self.month, self.day) == (instance.month, instance.day)

    # ADDITIONS AND SUBSTRACTIONS

    def add(
        self,
        years: int = 0,
        months: int = 0,
        weeks: int = 0,
        days: int = 0,
        hours: int = 0,
        minutes: int = 0,
        seconds: float = 0,
        microseconds: int = 0,
    ) -> Self:
        """
        Add a duration to the instance.

        If we're adding units of variable length (i.e., years, months),
        move forward from current time, otherwise move forward from utc, for accuracy
        when moving across DST boundaries.
        """
        units_of_variable_length = any([years, months, weeks, days])

        current_dt = datetime.datetime(
            self.year,
            self.month,
            self.day,
            self.hour,
            self.minute,
            self.second,
            self.microsecond,
        )
        if not units_of_variable_length:
            offset = self.utcoffset()
            if offset:
                current_dt = current_dt - offset

        dt = add_duration(
            current_dt,
            years=years,
            months=months,
            weeks=weeks,
            days=days,
            hours=hours,
            minutes=minutes,
            seconds=seconds,
            microseconds=microseconds,
        )

        if units_of_variable_length or self.tz is None:
            return self.__class__.create(
                dt.year,
                dt.month,
                dt.day,
                dt.hour,
                dt.minute,
                dt.second,
                dt.microsecond,
                tz=self.tz,
            )

        dt = datetime.datetime(
            dt.year,
            dt.month,
            dt.day,
            dt.hour,
            dt.minute,
            dt.second,
            dt.microsecond,
            tzinfo=UTC,
        )

        dt = self.tz.convert(dt)

        return self.__class__(
            dt.year,
            dt.month,
            dt.day,
            dt.hour,
            dt.minute,
            dt.second,
            dt.microsecond,
            tzinfo=self.tz,
            fold=dt.fold,
        )

    def subtract(
        self,
        years: int = 0,
        months: int = 0,
        weeks: int = 0,
        days: int = 0,
        hours: int = 0,
        minutes: int = 0,
        seconds: float = 0,
        microseconds: int = 0,
    ) -> Self:
        """
        Remove duration from the instance.
        """
        return self.add(
            years=-years,
            months=-months,
            weeks=-weeks,
            days=-days,
            hours=-hours,
            minutes=-minutes,
            seconds=-seconds,
            microseconds=-microseconds,
        )

    # Adding a final underscore to the method name
    # to avoid errors for PyPy which already defines
    # a _add_timedelta method
    def _add_timedelta_(self, delta: datetime.timedelta) -> Self:
        """
        Add timedelta duration to the instance.
        """
        if isinstance(delta, pendulum.Interval):
            return self.add(
                years=delta.years,
                months=delta.months,
                weeks=delta.weeks,
                days=delta.remaining_days,
                hours=delta.hours,
                minutes=delta.minutes,
                seconds=delta.remaining_seconds,
                microseconds=delta.microseconds,
            )
        elif isinstance(delta, pendulum.Duration):
            return self.add(**delta._signature)  # type: ignore[attr-defined]

        return self.add(seconds=delta.total_seconds())

    def _subtract_timedelta(self, delta: datetime.timedelta) -> Self:
        """
        Remove timedelta duration from the instance.
        """
        if isinstance(delta, pendulum.Duration):
            return self.subtract(
                years=delta.years, months=delta.months, seconds=delta._total
            )

        return self.subtract(seconds=delta.total_seconds())

    # DIFFERENCES

    def diff(  # type: ignore[override]
        self, dt: datetime.datetime | None = None, abs: bool = True
    ) -> Interval[datetime.datetime]:
        """
        Returns the difference between two DateTime objects represented as an Interval.
        """
        if dt is None:
            dt = self.now(self.tz)

        return Interval(self, dt, absolute=abs)

    def diff_for_humans(  # type: ignore[override]
        self,
        other: DateTime | None = None,
        absolute: bool = False,
        locale: str | None = None,
    ) -> str:
        """
        Get the difference in a human readable format in the current locale.

        When comparing a value in the past to default now:
        1 day ago
        5 months ago

        When comparing a value in the future to default now:
        1 day from now
        5 months from now

        When comparing a value in the past to another value:
        1 day before
        5 months before

        When comparing a value in the future to another value:
        1 day after
        5 months after
        """
        is_now = other is None

        if is_now:
            other = self.now()

        diff = self.diff(other)

        return pendulum.format_diff(diff, is_now, absolute, locale)

    # Modifiers
    def start_of(self, unit: str) -> Self:
        """
        Returns a copy of the instance with the time reset
        with the following rules:

        * second: microsecond set to 0
        * minute: second and microsecond set to 0
        * hour: minute, second and microsecond set to 0
        * day: time to 00:00:00
        * week: date to first day of the week and time to 00:00:00
        * month: date to first day of the month and time to 00:00:00
        * year: date to first day of the year and time to 00:00:00
        * decade: date to first day of the decade and time to 00:00:00
        * century: date to first day of century and time to 00:00:00
        """
        if unit not in self._MODIFIERS_VALID_UNITS:
            raise ValueError(f'Invalid unit "{unit}" for start_of()')

        return cast("Self", getattr(self, f"_start_of_{unit}")())

    def end_of(self, unit: str) -> Self:
        """
        Returns a copy of the instance with the time reset
        with the following rules:

        * second: microsecond set to 999999
        * minute: second set to 59 and microsecond set to 999999
        * hour: minute and second set to 59 and microsecond set to 999999
        * day: time to 23:59:59.999999
        * week: date to last day of the week and time to 23:59:59.999999
        * month: date to last day of the month and time to 23:59:59.999999
        * year: date to last day of the year and time to 23:59:59.999999
        * decade: date to last day of the decade and time to 23:59:59.999999
        * century: date to last day of century and time to 23:59:59.999999
        """
        if unit not in self._MODIFIERS_VALID_UNITS:
            raise ValueError(f'Invalid unit "{unit}" for end_of()')

        return cast("Self", getattr(self, f"_end_of_{unit}")())

    def _start_of_second(self) -> Self:
        """
        Reset microseconds to 0.
        """
        return self.set(microsecond=0)

    def _end_of_second(self) -> Self:
        """
        Set microseconds to 999999.
        """
        return self.set(microsecond=999999)

    def _start_of_minute(self) -> Self:
        """
        Reset seconds and microseconds to 0.
        """
        return self.set(second=0, microsecond=0)

    def _end_of_minute(self) -> Self:
        """
        Set seconds to 59 and microseconds to 999999.
        """
        return self.set(second=59, microsecond=999999)

    def _start_of_hour(self) -> Self:
        """
        Reset minutes, seconds and microseconds to 0.
        """
        return self.set(minute=0, second=0, microsecond=0)

    def _end_of_hour(self) -> Self:
        """
        Set minutes and seconds to 59 and microseconds to 999999.
        """
        return self.set(minute=59, second=59, microsecond=999999)

    def _start_of_day(self) -> Self:
        """
        Reset the time to 00:00:00.
        """
        return self.at(0, 0, 0, 0)

    def _end_of_day(self) -> Self:
        """
        Reset the time to 23:59:59.999999.
        """
        return self.at(23, 59, 59, 999999)

    def _start_of_month(self) -> Self:
        """
        Reset the date to the first day of the month and the time to 00:00:00.
        """
        return self.set(self.year, self.month, 1, 0, 0, 0, 0)

    def _end_of_month(self) -> Self:
        """
        Reset the date to the last day of the month
        and the time to 23:59:59.999999.
        """
        return self.set(self.year, self.month, self.days_in_month, 23, 59, 59, 999999)

    def _start_of_year(self) -> Self:
        """
        Reset the date to the first day of the year and the time to 00:00:00.
        """
        return self.set(self.year, 1, 1, 0, 0, 0, 0)

    def _end_of_year(self) -> Self:
        """
        Reset the date to the last day of the year
        and the time to 23:59:59.999999.
        """
        return self.set(self.year, 12, 31, 23, 59, 59, 999999)

    def _start_of_decade(self) -> Self:
        """
        Reset the date to the first day of the decade
        and the time to 00:00:00.
        """
        year = self.year - self.year % YEARS_PER_DECADE
        return self.set(year, 1, 1, 0, 0, 0, 0)

    def _end_of_decade(self) -> Self:
        """
        Reset the date to the last day of the decade
        and the time to 23:59:59.999999.
        """
        year = self.year - self.year % YEARS_PER_DECADE + YEARS_PER_DECADE - 1

        return self.set(year, 12, 31, 23, 59, 59, 999999)

    def _start_of_century(self) -> Self:
        """
        Reset the date to the first day of the century
        and the time to 00:00:00.
        """
        year = self.year - 1 - (self.year - 1) % YEARS_PER_CENTURY + 1

        return self.set(year, 1, 1, 0, 0, 0, 0)

    def _end_of_century(self) -> Self:
        """
        Reset the date to the last day of the century
        and the time to 23:59:59.999999.
        """
        year = self.year - 1 - (self.year - 1) % YEARS_PER_CENTURY + YEARS_PER_CENTURY

        return self.set(year, 12, 31, 23, 59, 59, 999999)

    def _start_of_week(self) -> Self:
        """
        Reset the date to the first day of the week
        and the time to 00:00:00.
        """
        dt = self

        if self.day_of_week != pendulum._WEEK_STARTS_AT:
            dt = self.previous(pendulum._WEEK_STARTS_AT)

        return dt.start_of("day")

    def _end_of_week(self) -> Self:
        """
        Reset the date to the last day of the week
        and the time to 23:59:59.
        """
        dt = self

        if self.day_of_week != pendulum._WEEK_ENDS_AT:
            dt = self.next(pendulum._WEEK_ENDS_AT)

        return dt.end_of("day")

    def next(self, day_of_week: WeekDay | None = None, keep_time: bool = False) -> Self:
        """
        Modify to the next occurrence of a given day of the week.
        If no day_of_week is provided, modify to the next occurrence
        of the current day of the week.  Use the supplied consts
        to indicate the desired day_of_week, ex. DateTime.MONDAY.
        """
        if day_of_week is None:
            day_of_week = self.day_of_week

        if day_of_week < WeekDay.MONDAY or day_of_week > WeekDay.SUNDAY:
            raise ValueError("Invalid day of week")

        dt = self if keep_time else self.start_of("day")

        dt = dt.add(days=1)
        while dt.day_of_week != day_of_week:
            dt = dt.add(days=1)

        return dt

    def previous(
        self, day_of_week: WeekDay | None = None, keep_time: bool = False
    ) -> Self:
        """
        Modify to the previous occurrence of a given day of the week.
        If no day_of_week is provided, modify to the previous occurrence
        of the current day of the week.  Use the supplied consts
        to indicate the desired day_of_week, ex. DateTime.MONDAY.
        """
        if day_of_week is None:
            day_of_week = self.day_of_week

        if day_of_week < WeekDay.MONDAY or day_of_week > WeekDay.SUNDAY:
            raise ValueError("Invalid day of week")

        dt = self if keep_time else self.start_of("day")

        dt = dt.subtract(days=1)
        while dt.day_of_week != day_of_week:
            dt = dt.subtract(days=1)

        return dt

    def first_of(self, unit: str, day_of_week: WeekDay | None = None) -> Self:
        """
        Returns an instance set to the first occurrence
        of a given day of the week in the current unit.
        If no day_of_week is provided, modify to the first day of the unit.
        Use the supplied consts to indicate the desired day_of_week,
        ex. DateTime.MONDAY.

        Supported units are month, quarter and year.
        """
        if unit not in ["month", "quarter", "year"]:
            raise ValueError(f'Invalid unit "{unit}" for first_of()')

        return cast("Self", getattr(self, f"_first_of_{unit}")(day_of_week))

    def last_of(self, unit: str, day_of_week: WeekDay | None = None) -> Self:
        """
        Returns an instance set to the last occurrence
        of a given day of the week in the current unit.
        If no day_of_week is provided, modify to the last day of the unit.
        Use the supplied consts to indicate the desired day_of_week,
        ex. DateTime.MONDAY.

        Supported units are month, quarter and year.
        """
        if unit not in ["month", "quarter", "year"]:
            raise ValueError(f'Invalid unit "{unit}" for first_of()')

        return cast("Self", getattr(self, f"_last_of_{unit}")(day_of_week))

    def nth_of(self, unit: str, nth: int, day_of_week: WeekDay) -> Self:
        """
        Returns a new instance set to the given occurrence
        of a given day of the week in the current unit.
        If the calculated occurrence is outside the scope of the current unit,
        then raise an error. Use the supplied consts
        to indicate the desired day_of_week, ex. DateTime.MONDAY.

        Supported units are month, quarter and year.
        """
        if unit not in ["month", "quarter", "year"]:
            raise ValueError(f'Invalid unit "{unit}" for first_of()')

        dt = cast("Optional[Self]", getattr(self, f"_nth_of_{unit}")(nth, day_of_week))
        if not dt:
            raise PendulumException(
                f"Unable to find occurrence {nth}"
                f" of {WeekDay(day_of_week).name.capitalize()} in {unit}"
            )

        return dt

    def _first_of_month(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the first occurrence of a given day of the week
        in the current month. If no day_of_week is provided,
        modify to the first day of the month. Use the supplied consts
        to indicate the desired day_of_week, ex. DateTime.MONDAY.
        """
        dt = self.start_of("day")

        if day_of_week is None:
            return dt.set(day=1)

        month = calendar.monthcalendar(dt.year, dt.month)

        calendar_day = day_of_week

        if month[0][calendar_day] > 0:
            day_of_month = month[0][calendar_day]
        else:
            day_of_month = month[1][calendar_day]

        return dt.set(day=day_of_month)

    def _last_of_month(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the last occurrence of a given day of the week
        in the current month. If no day_of_week is provided,
        modify to the last day of the month. Use the supplied consts
        to indicate the desired day_of_week, ex. DateTime.MONDAY.
        """
        dt = self.start_of("day")

        if day_of_week is None:
            return dt.set(day=self.days_in_month)

        month = calendar.monthcalendar(dt.year, dt.month)

        calendar_day = day_of_week

        if month[-1][calendar_day] > 0:
            day_of_month = month[-1][calendar_day]
        else:
            day_of_month = month[-2][calendar_day]

        return dt.set(day=day_of_month)

    def _nth_of_month(
        self, nth: int, day_of_week: WeekDay | None = None
    ) -> Self | None:
        """
        Modify to the given occurrence of a given day of the week
        in the current month. If the calculated occurrence is outside,
        the scope of the current month, then return False and no
        modifications are made. Use the supplied consts
        to indicate the desired day_of_week, ex. DateTime.MONDAY.
        """
        if nth == 1:
            return self.first_of("month", day_of_week)

        dt = self.first_of("month")
        check = dt.format("%Y-%M")
        for _ in range(nth - (1 if dt.day_of_week == day_of_week else 0)):
            dt = dt.next(day_of_week)

        if dt.format("%Y-%M") == check:
            return self.set(day=dt.day).start_of("day")

        return None

    def _first_of_quarter(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the first occurrence of a given day of the week
        in the current quarter. If no day_of_week is provided,
        modify to the first day of the quarter. Use the supplied consts
        to indicate the desired day_of_week, ex. DateTime.MONDAY.
        """
        return self.on(self.year, self.quarter * 3 - 2, 1).first_of(
            "month", day_of_week
        )

    def _last_of_quarter(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the last occurrence of a given day of the week
        in the current quarter. If no day_of_week is provided,
        modify to the last day of the quarter. Use the supplied consts
        to indicate the desired day_of_week, ex. DateTime.MONDAY.
        """
        return self.on(self.year, self.quarter * 3, 1).last_of("month", day_of_week)

    def _nth_of_quarter(
        self, nth: int, day_of_week: WeekDay | None = None
    ) -> Self | None:
        """
        Modify to the given occurrence of a given day of the week
        in the current quarter. If the calculated occurrence is outside,
        the scope of the current quarter, then return False and no
        modifications are made. Use the supplied consts
        to indicate the desired day_of_week, ex. DateTime.MONDAY.
        """
        if nth == 1:
            return self.first_of("quarter", day_of_week)

        dt = self.set(day=1, month=self.quarter * 3)
        last_month = dt.month
        year = dt.year
        dt = dt.first_of("quarter")
        for _ in range(nth - (1 if dt.day_of_week == day_of_week else 0)):
            dt = dt.next(day_of_week)

        if last_month < dt.month or year != dt.year:
            return None

        return self.on(self.year, dt.month, dt.day).start_of("day")

    def _first_of_year(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the first occurrence of a given day of the week
        in the current year. If no day_of_week is provided,
        modify to the first day of the year. Use the supplied consts
        to indicate the desired day_of_week, ex. DateTime.MONDAY.
        """
        return self.set(month=1).first_of("month", day_of_week)

    def _last_of_year(self, day_of_week: WeekDay | None = None) -> Self:
        """
        Modify to the last occurrence of a given day of the week
        in the current year. If no day_of_week is provided,
        modify to the last day of the year. Use the supplied consts
        to indicate the desired day_of_week, ex. DateTime.MONDAY.
        """
        return self.set(month=MONTHS_PER_YEAR).last_of("month", day_of_week)

    def _nth_of_year(self, nth: int, day_of_week: WeekDay | None = None) -> Self | None:
        """
        Modify to the given occurrence of a given day of the week
        in the current year. If the calculated occurrence is outside,
        the scope of the current year, then return False and no
        modifications are made. Use the supplied consts
        to indicate the desired day_of_week, ex. DateTime.MONDAY.
        """
        if nth == 1:
            return self.first_of("year", day_of_week)

        dt = self.first_of("year")
        year = dt.year
        for _ in range(nth - (1 if dt.day_of_week == day_of_week else 0)):
            dt = dt.next(day_of_week)

        if year != dt.year:
            return None

        return self.on(self.year, dt.month, dt.day).start_of("day")

    def average(  # type: ignore[override]
        self, dt: datetime.datetime | None = None
    ) -> Self:
        """
        Modify the current instance to the average
        of a given instance (default now) and the current instance.
        """
        if dt is None:
            dt = self.now(self.tz)

        diff = self.diff(dt, False)
        return self.add(
            microseconds=(diff.in_seconds() * 1000000 + diff.microseconds) // 2
        )

    @overload  # type: ignore[override]
    def __sub__(self, other: datetime.timedelta) -> Self: ...

    @overload
    def __sub__(self, other: DateTime) -> Interval[datetime.datetime]: ...

    def __sub__(
        self, other: datetime.datetime | datetime.timedelta
    ) -> Self | Interval[datetime.datetime]:
        if isinstance(other, datetime.timedelta):
            return self._subtract_timedelta(other)

        if not isinstance(other, datetime.datetime):
            return NotImplemented

        if not isinstance(other, self.__class__):
            if other.tzinfo is None:
                other = pendulum.naive(
                    other.year,
                    other.month,
                    other.day,
                    other.hour,
                    other.minute,
                    other.second,
                    other.microsecond,
                )
            else:
                other = self.instance(other)

        return other.diff(self, False)

    def __rsub__(self, other: datetime.datetime) -> Interval[datetime.datetime]:
        if not isinstance(other, datetime.datetime):
            return NotImplemented

        if not isinstance(other, self.__class__):
            if other.tzinfo is None:
                other = pendulum.naive(
                    other.year,
                    other.month,
                    other.day,
                    other.hour,
                    other.minute,
                    other.second,
                    other.microsecond,
                )
            else:
                other = self.instance(other)

        return self.diff(other, False)

    def __add__(self, other: datetime.timedelta) -> Self:
        if not isinstance(other, datetime.timedelta):
            return NotImplemented

        caller = traceback.extract_stack(limit=2)[0].name
        if caller == "astimezone":
            return super().__add__(other)

        return self._add_timedelta_(other)

    def __radd__(self, other: datetime.timedelta) -> Self:
        return self.__add__(other)

    # Native methods override

    @classmethod
    def fromtimestamp(cls, t: float, tz: datetime.tzinfo | None = None) -> Self:
        tzinfo = pendulum._safe_timezone(tz)

        return cls.instance(datetime.datetime.fromtimestamp(t, tz=tzinfo), tz=tzinfo)

    @classmethod
    def utcfromtimestamp(cls, t: float) -> Self:
        return cls.instance(datetime.datetime.utcfromtimestamp(t), tz=None)

    @classmethod
    def fromordinal(cls, n: int) -> Self:
        return cls.instance(datetime.datetime.fromordinal(n), tz=None)

    @classmethod
    def combine(
        cls,
        date: datetime.date,
        time: datetime.time,
        tzinfo: datetime.tzinfo | None = None,
    ) -> Self:
        return cls.instance(datetime.datetime.combine(date, time), tz=tzinfo)

    def astimezone(self, tz: datetime.tzinfo | None = None) -> Self:
        dt = super().astimezone(tz)

        return self.__class__(
            dt.year,
            dt.month,
            dt.day,
            dt.hour,
            dt.minute,
            dt.second,
            dt.microsecond,
            fold=dt.fold,
            tzinfo=dt.tzinfo,
        )

    def replace(
        self,
        year: SupportsIndex | None = None,
        month: SupportsIndex | None = None,
        day: SupportsIndex | None = None,
        hour: SupportsIndex | None = None,
        minute: SupportsIndex | None = None,
        second: SupportsIndex | None = None,
        microsecond: SupportsIndex | None = None,
        tzinfo: bool | datetime.tzinfo | Literal[True] | None = True,
        fold: int | None = None,
    ) -> Self:
        if year is None:
            year = self.year
        if month is None:
            month = self.month
        if day is None:
            day = self.day
        if hour is None:
            hour = self.hour
        if minute is None:
            minute = self.minute
        if second is None:
            second = self.second
        if microsecond is None:
            microsecond = self.microsecond
        if tzinfo is True:
            tzinfo = self.tzinfo
        if fold is None:
            fold = self.fold

        if tzinfo is not None:
            tzinfo = pendulum._safe_timezone(tzinfo)

        return self.__class__.create(
            year,
            month,
            day,
            hour,
            minute,
            second,
            microsecond,
            tz=tzinfo,
            fold=fold,
        )

    def __getnewargs__(self) -> tuple[Self]:
        return (self,)

    def _getstate(
        self, protocol: SupportsIndex = 3
    ) -> tuple[int, int, int, int, int, int, int, datetime.tzinfo | None]:
        return (
            self.year,
            self.month,
            self.day,
            self.hour,
            self.minute,
            self.second,
            self.microsecond,
            self.tzinfo,
        )

    def __reduce__(
        self,
    ) -> tuple[
        type[Self],
        tuple[int, int, int, int, int, int, int, datetime.tzinfo | None],
    ]:
        return self.__reduce_ex__(2)

    def __reduce_ex__(
        self, protocol: SupportsIndex
    ) -> tuple[
        type[Self],
        tuple[int, int, int, int, int, int, int, datetime.tzinfo | None],
    ]:
        return self.__class__, self._getstate(protocol)

    def __deepcopy__(self, _: dict[int, Self]) -> Self:
        return self.__class__(
            self.year,
            self.month,
            self.day,
            self.hour,
            self.minute,
            self.second,
            self.microsecond,
            tzinfo=self.tz,
            fold=self.fold,
        )

    def _cmp(self, other: datetime.datetime, **kwargs: Any) -> int:
        # Fix for pypy which compares using this method
        # which would lead to infinite recursion if we didn't override
        dt = datetime.datetime(
            self.year,
            self.month,
            self.day,
            self.hour,
            self.minute,
            self.second,
            self.microsecond,
            tzinfo=self.tz,
            fold=self.fold,
        )

        return 0 if dt == other else 1 if dt > other else -1


DateTime.min = DateTime(1, 1, 1, 0, 0, tzinfo=UTC)
DateTime.max = DateTime(9999, 12, 31, 23, 59, 59, 999999, tzinfo=UTC)
DateTime.EPOCH = DateTime(1970, 1, 1, tzinfo=UTC)



from __future__ import annotations

import pickle
import zoneinfo

from copy import deepcopy
from datetime import date
from datetime import datetime
from datetime import time

import pytest

import pendulum

from pendulum import timezone
from pendulum.tz.timezone import Timezone


@pytest.fixture
def p():
    return pendulum.datetime(2016, 8, 27, 12, 34, 56, 123456, tz="Europe/Paris")


@pytest.fixture
def p1(p):
    return p.in_tz("America/New_York")


@pytest.fixture
def dt():
    tz = timezone("Europe/Paris")

    return tz.convert(datetime(2016, 8, 27, 12, 34, 56, 123456))


def test_timetuple(p, dt):
    assert dt.timetuple() == p.timetuple()


def test_utctimetuple(p, dt):
    assert dt.utctimetuple() == p.utctimetuple()


def test_date(p, dt):
    assert p.date() == dt.date()


def test_time(p, dt):
    assert p.time() == dt.time()


def test_timetz(p, dt):
    assert p.timetz() == dt.timetz()


def test_astimezone(p, dt, p1):
    assert p.astimezone(p1.tzinfo) == dt.astimezone(p1.tzinfo)


def test_ctime(p, dt):
    assert p.ctime() == dt.ctime()


def test_isoformat(p, dt):
    assert p.isoformat() == dt.isoformat()


def test_utcoffset(p, dt):
    assert p.utcoffset() == dt.utcoffset()


def test_tzname(p, dt):
    assert p.tzname() == dt.tzname()


def test_dst(p, dt):
    assert p.dst() == dt.dst()


def test_toordinal(p, dt):
    assert p.toordinal() == dt.toordinal()


def test_weekday(p, dt):
    assert p.weekday() == dt.weekday()


def test_isoweekday(p, dt):
    assert p.isoweekday() == dt.isoweekday()


def test_isocalendar(p, dt):
    assert p.isocalendar() == dt.isocalendar()


def test_fromtimestamp():
    p = pendulum.DateTime.fromtimestamp(0, pendulum.UTC)
    dt = datetime.fromtimestamp(0, pendulum.UTC)

    assert p == dt


def test_utcfromtimestamp():
    p = pendulum.DateTime.utcfromtimestamp(0)
    dt = datetime.utcfromtimestamp(0)

    assert p == dt


def test_fromordinal():
    assert datetime.fromordinal(730120) == pendulum.DateTime.fromordinal(730120)


def test_combine():
    p = pendulum.DateTime.combine(date(2016, 1, 1), time(1, 2, 3, 123456))
    dt = datetime.combine(date(2016, 1, 1), time(1, 2, 3, 123456))

    assert p == dt


def test_hash(p, dt):
    assert hash(p) == hash(dt)

    dt1 = pendulum.datetime(2016, 8, 27, 12, 34, 56, 123456, tz="Europe/Paris")
    dt2 = pendulum.datetime(2016, 8, 27, 12, 34, 56, 123456, tz="Europe/Paris")
    dt3 = pendulum.datetime(2016, 8, 27, 12, 34, 56, 123456, tz="America/Toronto")

    assert hash(dt1) == hash(dt2)
    assert hash(dt1) != hash(dt3)


def test_pickle():
    dt1 = pendulum.datetime(2016, 8, 27, 12, 34, 56, 123456, tz="Europe/Paris")
    s = pickle.dumps(dt1)
    dt2 = pickle.loads(s)

    assert dt1 == dt2


def test_pickle_with_integer_tzinfo():
    dt1 = pendulum.datetime(2016, 8, 27, 12, 34, 56, 123456, tz=0)
    s = pickle.dumps(dt1)
    dt2 = pickle.loads(s)

    assert dt1 == dt2


def test_proper_dst():
    dt = pendulum.datetime(1941, 7, 1, tz="Europe/Amsterdam")
    native_dt = datetime(1941, 7, 1, tzinfo=zoneinfo.ZoneInfo("Europe/Amsterdam"))

    assert dt.dst() == native_dt.dst()


def test_deepcopy():
    dt = pendulum.datetime(1941, 7, 1, tz="Europe/Amsterdam")

    assert dt == deepcopy(dt)


def test_deepcopy_on_transition():
    dt = pendulum.datetime(2023, 11, 5, 1, 0, 0, tz="US/Pacific")
    clone = deepcopy(dt)

    assert dt == clone
    assert dt.offset == clone.offset


def test_pickle_timezone():
    dt1 = pendulum.timezone("Europe/Amsterdam")
    s = pickle.dumps(dt1)
    dt2 = pickle.loads(s)

    assert isinstance(dt2, Timezone)

    dt1 = pendulum.timezone("UTC")
    s = pickle.dumps(dt1)
    dt2 = pickle.loads(s)

    assert isinstance(dt2, Timezone)



from __future__ import annotations

import pendulum


def test_week():
    start_date = pendulum.datetime(2012, 1, 1)
    interval = pendulum.interval(start=start_date, end=start_date.add(weeks=1))
    assert interval.in_words() == "1 week"


def test_week_and_day():
    start_date = pendulum.datetime(2012, 1, 1)
    interval = pendulum.interval(start=start_date, end=start_date.add(weeks=1, days=1))
    assert interval.in_words() == "1 week 1 day"


def test_all():
    start_date = pendulum.datetime(2012, 1, 1)
    interval = pendulum.interval(
        start=start_date,
        end=start_date.add(years=1, months=1, days=1, seconds=1, microseconds=1),
    )
    assert interval.in_words() == "1 year 1 month 1 day 1 second"


def test_in_french():
    start_date = pendulum.datetime(2012, 1, 1)
    interval = pendulum.interval(
        start=start_date,
        end=start_date.add(years=1, months=1, days=1, seconds=1, microseconds=1),
    )
    assert interval.in_words(locale="fr") == "1 an 1 mois 1 jour 1 seconde"


def test_singular_negative_values():
    start_date = pendulum.datetime(2012, 1, 1)
    interval = pendulum.interval(start=start_date, end=start_date.subtract(days=1))
    assert interval.in_words() == "-1 day"


def test_separator():
    start_date = pendulum.datetime(2012, 1, 1)
    interval = pendulum.interval(
        start=start_date,
        end=start_date.add(years=1, months=1, days=1, seconds=1, microseconds=1),
    )
    assert interval.in_words(separator=", ") == "1 year, 1 month, 1 day, 1 second"


def test_subseconds():
    start_date = pendulum.datetime(2012, 1, 1)
    interval = pendulum.interval(
        start=start_date, end=start_date.add(microseconds=123456)
    )
    assert interval.in_words() == "0.12 second"


def test_subseconds_with_seconds():
    start_date = pendulum.datetime(2012, 1, 1)
    interval = pendulum.interval(
        start=start_date, end=start_date.add(seconds=12, microseconds=123456)
    )
    assert interval.in_words() == "12 seconds"


def test_zero_interval():
    start_date = pendulum.datetime(2012, 1, 1)
    interval = pendulum.interval(start=start_date, end=start_date)
    assert interval.in_words() == "0 microseconds"



from __future__ import annotations

import pendulum


def test_year():
    d = pendulum.Date(1234, 5, 6)
    assert d.year == 1234


def test_month():
    d = pendulum.Date(1234, 5, 6)
    assert d.month == 5


def test_day():
    d = pendulum.Date(1234, 5, 6)
    assert d.day == 6


def test_day_of_week():
    d = pendulum.Date(2012, 5, 7)
    assert d.day_of_week == pendulum.MONDAY


def test_day_of_year():
    d = pendulum.Date(2015, 12, 31)
    assert d.day_of_year == 365
    d = pendulum.Date(2016, 12, 31)
    assert d.day_of_year == 366


def test_days_in_month():
    d = pendulum.Date(2012, 5, 7)
    assert d.days_in_month == 31


def test_age():
    d = pendulum.Date.today()
    assert d.age == 0
    assert d.add(years=1).age == -1
    assert d.subtract(years=1).age == 1


def test_is_leap_year():
    assert pendulum.Date(2012, 1, 1).is_leap_year()
    assert not pendulum.Date(2011, 1, 1).is_leap_year()


def test_is_long_year():
    assert pendulum.Date(2015, 1, 1).is_long_year()
    assert not pendulum.Date(2016, 1, 1).is_long_year()


def test_week_of_month():
    assert pendulum.Date(2012, 9, 30).week_of_month == 5
    assert pendulum.Date(2012, 9, 28).week_of_month == 5
    assert pendulum.Date(2012, 9, 20).week_of_month == 4
    assert pendulum.Date(2012, 9, 8).week_of_month == 2
    assert pendulum.Date(2012, 9, 1).week_of_month == 1
    assert pendulum.date(2020, 1, 1).week_of_month == 1
    assert pendulum.date(2020, 1, 7).week_of_month == 2
    assert pendulum.date(2020, 1, 14).week_of_month == 3


def test_week_of_year_first_week():
    assert pendulum.Date(2012, 1, 1).week_of_year == 52
    assert pendulum.Date(2012, 1, 2).week_of_year == 1


def test_week_of_year_last_week():
    assert pendulum.Date(2012, 12, 30).week_of_year == 52
    assert pendulum.Date(2012, 12, 31).week_of_year == 1


def test_is_future():
    d = pendulum.Date.today()
    assert not d.is_future()
    d = d.add(days=1)
    assert d.is_future()


def test_is_past():
    d = pendulum.Date.today()
    assert not d.is_past()
    d = d.subtract(days=1)
    assert d.is_past()



from __future__ import annotations

import pendulum


def test_week():
    assert pendulum.duration(days=364).in_words() == "52 weeks"
    assert pendulum.duration(days=7).in_words() == "1 week"


def test_week_to_string():
    assert str(pendulum.duration(days=364)) == "52 weeks"
    assert str(pendulum.duration(days=7)) == "1 week"


def test_weeks_and_day():
    assert pendulum.duration(days=365).in_words() == "52 weeks 1 day"


def test_all():
    pi = pendulum.duration(
        years=2, months=3, days=1177, seconds=7284, microseconds=1000000
    )

    expected = "2 years 3 months 168 weeks 1 day 2 hours 1 minute 25 seconds"
    assert pi.in_words() == expected


def test_in_french():
    pi = pendulum.duration(
        years=2, months=3, days=1177, seconds=7284, microseconds=1000000
    )

    expected = "2 ans 3 mois 168 semaines 1 jour 2 heures 1 minute 25 secondes"
    assert pi.in_words(locale="fr") == expected


def test_repr():
    pi = pendulum.duration(
        years=2, months=3, days=1177, seconds=7284, microseconds=1000000
    )

    expected = (
        "Duration(years=2, months=3, weeks=168, days=1, hours=2, minutes=1, seconds=25)"
    )
    assert repr(pi) == expected


def test_singular_negative_values():
    pi = pendulum.duration(days=-1)

    assert pi.in_words() == "-1 day"


def test_separator():
    pi = pendulum.duration(days=1177, seconds=7284, microseconds=1000000)

    expected = "168 weeks, 1 day, 2 hours, 1 minute, 25 seconds"
    assert pi.in_words(separator=", ") == expected


def test_subseconds():
    pi = pendulum.duration(microseconds=123456)

    assert pi.in_words() == "0.12 second"


def test_subseconds_with_seconds():
    pi = pendulum.duration(seconds=12, microseconds=123456)

    assert pi.in_words() == "12 seconds"


def test_duration_with_all_zero_values():
    pi = pendulum.duration()

    assert pi.in_words() == "0 microseconds"



from __future__ import annotations

from datetime import date
from datetime import datetime
from datetime import time

import pytz

from dateutil import tz

import pendulum

from pendulum import _safe_timezone
from pendulum import timezone
from pendulum.tz.timezone import Timezone


def test_instance_with_naive_datetime_defaults_to_utc() -> None:
    now = pendulum.instance(datetime.now())
    assert now.timezone_name == "UTC"


def test_instance_with_aware_datetime() -> None:
    now = pendulum.instance(datetime.now(timezone("Europe/Paris")))
    assert now.timezone_name == "Europe/Paris"


def test_instance_with_aware_datetime_pytz() -> None:
    now = pendulum.instance(datetime.now(pytz.timezone("Europe/Paris")))
    assert now.timezone_name == "Europe/Paris"


def test_instance_with_aware_datetime_any_tzinfo() -> None:
    dt = datetime(2016, 8, 7, 12, 34, 56, tzinfo=tz.gettz("Europe/Paris"))
    now = pendulum.instance(dt)
    assert now.timezone_name == "+02:00"


def test_instance_with_date() -> None:
    dt = pendulum.instance(date(2022, 12, 23))

    assert isinstance(dt, pendulum.Date)


def test_instance_with_naive_time() -> None:
    dt = pendulum.instance(time(12, 34, 56, 123456))

    assert isinstance(dt, pendulum.Time)


def test_instance_with_aware_time() -> None:
    dt = pendulum.instance(time(12, 34, 56, 123456, tzinfo=timezone("Europe/Paris")))

    assert isinstance(dt, pendulum.Time)
    assert isinstance(dt.tzinfo, Timezone)
    assert dt.tzinfo.name == "Europe/Paris"


def test_safe_timezone_with_tzinfo_objects() -> None:
    tz = _safe_timezone(pytz.timezone("Europe/Paris"))

    assert isinstance(tz, Timezone)
    assert tz.name == "Europe/Paris"



from __future__ import annotations

import pendulum


locale = "tr"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "1 saniye önce"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "2 saniye önce"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "1 dakika önce"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "2 dakika önce"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "1 saat önce"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "2 saat önce"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "1 gün önce"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "2 gün önce"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "1 hafta önce"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "2 hafta önce"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "1 ay önce"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "2 ay önce"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "1 yıl önce"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "2 yıl önce"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "1 saniye sonra"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "1 saniye sonra"
    assert d2.diff_for_humans(d, locale=locale) == "1 saniye önce"

    assert d.diff_for_humans(d2, True, locale=locale) == "1 saniye"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "2 saniye"



from __future__ import annotations

import pendulum


locale = "es"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "hace unos segundos"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "hace unos segundos"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "hace 1 minuto"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "hace 2 minutos"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "hace 1 hora"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "hace 2 horas"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "hace 1 día"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "hace 2 días"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "hace 1 semana"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "hace 2 semanas"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "hace 1 mes"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "hace 2 meses"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "hace 1 año"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "hace 2 años"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "dentro de unos segundos"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "unos segundos después"
    assert d2.diff_for_humans(d, locale=locale) == "unos segundos antes"

    assert d.diff_for_humans(d2, True, locale=locale) == "unos segundos"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "unos segundos"



from __future__ import annotations

import pendulum


locale = "fr"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "il y a quelques secondes"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "il y a quelques secondes"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "il y a 1 minute"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "il y a 2 minutes"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "il y a 1 heure"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "il y a 2 heures"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "il y a 1 jour"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "il y a 2 jours"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "il y a 1 semaine"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "il y a 2 semaines"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "il y a 1 mois"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "il y a 2 mois"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "il y a 1 an"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "il y a 2 ans"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "dans quelques secondes"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "quelques secondes après"
    assert d2.diff_for_humans(d, locale=locale) == "quelques secondes avant"

    assert d.diff_for_humans(d2, True, locale=locale) == "quelques secondes"
    assert (
        d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "quelques secondes"
    )


def test_format():
    d = pendulum.datetime(2016, 8, 28, 7, 3, 6, 123456)
    assert d.format("dddd", locale=locale) == "dimanche"
    assert d.format("ddd", locale=locale) == "dim."
    assert d.format("MMMM", locale=locale) == "août"
    assert d.format("MMM", locale=locale) == "août"
    assert d.format("A", locale=locale) == "AM"
    assert d.format("Do", locale=locale) == "28e"

    assert d.format("LT", locale=locale) == "07:03"
    assert d.format("LTS", locale=locale) == "07:03:06"
    assert d.format("L", locale=locale) == "28/08/2016"
    assert d.format("LL", locale=locale) == "28 août 2016"
    assert d.format("LLL", locale=locale) == "28 août 2016 07:03"
    assert d.format("LLLL", locale=locale) == "dimanche 28 août 2016 07:03"



from __future__ import annotations

import pendulum

from pendulum import Time


def test_diff_in_hours_positive():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.add(hours=2).add(seconds=3672)).in_hours() == 3


def test_diff_in_hours_negative_with_sign():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.subtract(hours=2).add(seconds=3600), False).in_hours() == -1


def test_diff_in_hours_negative_no_sign():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.subtract(hours=2).add(seconds=3600)).in_hours() == 1


def test_diff_in_hours_vs_default_now():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.subtract(hours=2).diff().in_hours() == 2


def test_diff_in_hours_ensure_is_truncated():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.add(hours=2).add(seconds=5401)).in_hours() == 3


def test_diff_in_minutes_positive():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.add(hours=1).add(minutes=2)).in_minutes() == 62


def test_diff_in_minutes_positive_big():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.add(hours=25).add(minutes=2)).in_minutes() == 62


def test_diff_in_minutes_negative_with_sign():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.subtract(hours=1).add(minutes=2), False).in_minutes() == -58


def test_diff_in_minutes_negative_no_sign():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.subtract(hours=1).add(minutes=2)).in_minutes() == 58


def test_diff_in_minutes_vs_default_now():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.subtract(hours=1).diff().in_minutes() == 60


def test_diff_in_minutes_ensure_is_truncated():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.add(minutes=1).add(seconds=59)).in_minutes() == 1


def test_diff_in_seconds_positive():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.add(minutes=1).add(seconds=2)).in_seconds() == 62


def test_diff_in_seconds_positive_big():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.add(hours=2).add(seconds=2)).in_seconds() == 7202


def test_diff_in_seconds_negative_with_sign():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.subtract(minutes=1).add(seconds=2), False).in_seconds() == -58


def test_diff_in_seconds_negative_no_sign():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.subtract(minutes=1).add(seconds=2)).in_seconds() == 58


def test_diff_in_seconds_vs_default_now():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.subtract(hours=1).diff().in_seconds() == 3600


def test_diff_in_seconds_ensure_is_truncated():
    dt = Time(12, 34, 56)
    assert dt.diff(dt.add(seconds=1.9)).in_seconds() == 1


def test_diff_for_humans_now_and_second():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans() == "a few seconds ago"


def test_diff_for_humans_now_and_seconds():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.subtract(seconds=2).diff_for_humans() == "a few seconds ago"


def test_diff_for_humans_now_and_nearly_minute():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.subtract(seconds=59).diff_for_humans() == "59 seconds ago"


def test_diff_for_humans_now_and_minute():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.subtract(minutes=1).diff_for_humans() == "1 minute ago"


def test_diff_for_humans_now_and_minutes():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.subtract(minutes=2).diff_for_humans() == "2 minutes ago"


def test_diff_for_humans_now_and_nearly_hour():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.subtract(minutes=59).diff_for_humans() == "59 minutes ago"


def test_diff_for_humans_now_and_hour():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.subtract(hours=1).diff_for_humans() == "1 hour ago"


def test_diff_for_humans_now_and_hours():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.subtract(hours=2).diff_for_humans() == "2 hours ago"


def test_diff_for_humans_now_and_future_second():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.add(seconds=1).diff_for_humans() == "in a few seconds"


def test_diff_for_humans_now_and_future_seconds():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.add(seconds=2).diff_for_humans() == "in a few seconds"


def test_diff_for_humans_now_and_nearly_future_minute():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.add(seconds=59).diff_for_humans() == "in 59 seconds"


def test_diff_for_humans_now_and_future_minute():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.add(minutes=1).diff_for_humans() == "in 1 minute"


def test_diff_for_humans_now_and_future_minutes():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.add(minutes=2).diff_for_humans() == "in 2 minutes"


def test_diff_for_humans_now_and_nearly_future_hour():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.add(minutes=59).diff_for_humans() == "in 59 minutes"


def test_diff_for_humans_now_and_future_hour():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.add(hours=1).diff_for_humans() == "in 1 hour"


def test_diff_for_humans_now_and_future_hours():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.add(hours=2).diff_for_humans() == "in 2 hours"


def test_diff_for_humans_other_and_second():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.add(seconds=1)) == "a few seconds before"


def test_diff_for_humans_other_and_seconds():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.add(seconds=2)) == "a few seconds before"


def test_diff_for_humans_other_and_nearly_minute():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.add(seconds=59)) == "59 seconds before"


def test_diff_for_humans_other_and_minute():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.add(minutes=1)) == "1 minute before"


def test_diff_for_humans_other_and_minutes():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.add(minutes=2)) == "2 minutes before"


def test_diff_for_humans_other_and_nearly_hour():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.add(minutes=59)) == "59 minutes before"


def test_diff_for_humans_other_and_hour():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.add(hours=1)) == "1 hour before"


def test_diff_for_humans_other_and_hours():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.add(hours=2)) == "2 hours before"


def test_diff_for_humans_other_and_future_second():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.subtract(seconds=1)) == "a few seconds after"


def test_diff_for_humans_other_and_future_seconds():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.subtract(seconds=2)) == "a few seconds after"


def test_diff_for_humans_other_and_nearly_future_minute():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.subtract(seconds=59)) == "59 seconds after"


def test_diff_for_humans_other_and_future_minute():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.subtract(minutes=1)) == "1 minute after"


def test_diff_for_humans_other_and_future_minutes():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.subtract(minutes=2)) == "2 minutes after"


def test_diff_for_humans_other_and_nearly_future_hour():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.subtract(minutes=59)) == "59 minutes after"


def test_diff_for_humans_other_and_future_hour():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.subtract(hours=1)) == "1 hour after"


def test_diff_for_humans_other_and_future_hours():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.subtract(hours=2)) == "2 hours after"


def test_diff_for_humans_absolute_seconds():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.subtract(seconds=59), True) == "59 seconds"
        now = pendulum.now().time()

        assert now.diff_for_humans(now.add(seconds=59), True) == "59 seconds"


def test_diff_for_humans_absolute_minutes():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.subtract(minutes=30), True) == "30 minutes"
        now = pendulum.now().time()

        assert now.diff_for_humans(now.add(minutes=30), True) == "30 minutes"


def test_diff_for_humans_absolute_hours():
    with pendulum.travel_to(pendulum.today().at(12, 34, 56)):
        now = pendulum.now().time()

        assert now.diff_for_humans(now.subtract(hours=3), True) == "3 hours"
        now = pendulum.now().time()

        assert now.diff_for_humans(now.add(hours=3), True) == "3 hours"



from __future__ import annotations

from datetime import datetime

import pendulum

from tests.conftest import assert_datetime


def test_with_datetimes():
    dt1 = datetime(2000, 1, 1)
    dt2 = datetime(2000, 1, 31)
    p = pendulum.interval(dt1, dt2)

    assert isinstance(p.start, pendulum.DateTime)
    assert isinstance(p.end, pendulum.DateTime)
    assert_datetime(p.start, 2000, 1, 1)
    assert_datetime(p.end, 2000, 1, 31)


def test_with_pendulum():
    dt1 = pendulum.DateTime(2000, 1, 1)
    dt2 = pendulum.DateTime(2000, 1, 31)
    p = pendulum.interval(dt1, dt2)

    assert_datetime(p.start, 2000, 1, 1)
    assert_datetime(p.end, 2000, 1, 31)


def test_inverted():
    dt1 = pendulum.DateTime(2000, 1, 1)
    dt2 = pendulum.DateTime(2000, 1, 31)
    p = pendulum.interval(dt2, dt1)

    assert_datetime(p.start, 2000, 1, 31)
    assert_datetime(p.end, 2000, 1, 1)


def test_inverted_and_absolute():
    dt1 = pendulum.DateTime(2000, 1, 1)
    dt2 = pendulum.DateTime(2000, 1, 31)
    p = pendulum.interval(dt2, dt1, True)

    assert_datetime(p.start, 2000, 1, 1)
    assert_datetime(p.end, 2000, 1, 31)


def test_accuracy():
    dt1 = pendulum.DateTime(2000, 11, 20)
    dt2 = pendulum.DateTime(2000, 11, 25)
    dt3 = pendulum.DateTime(2016, 11, 5)
    p1 = pendulum.interval(dt1, dt3)
    p2 = pendulum.interval(dt2, dt3)

    assert p1.years == 15
    assert p1.in_years() == 15
    assert p1.months == 11
    assert p1.in_months() == 191
    assert p1.days == 5829
    assert p1.remaining_days == 2
    assert p1.in_days() == 5829

    assert p2.years == 15
    assert p2.in_years() == 15
    assert p2.months == 11
    assert p2.in_months() == 191
    assert p2.days == 5824
    assert p2.remaining_days == 4
    assert p2.in_days() == 5824


def test_dst_transition():
    start = pendulum.datetime(2017, 3, 7, tz="America/Toronto")
    end = start.add(days=6)
    interval = end - start

    assert interval.days == 5
    assert interval.seconds == 82800

    assert interval.remaining_days == 6
    assert interval.hours == 0
    assert interval.remaining_seconds == 0

    assert interval.in_days() == 6
    assert interval.in_hours() == 5 * 24 + 23


def test_timedelta_behavior():
    dt1 = pendulum.DateTime(2000, 11, 20, 1)
    dt2 = pendulum.DateTime(2000, 11, 25, 2)
    dt3 = pendulum.DateTime(2016, 11, 5, 3)

    p1 = pendulum.interval(dt1, dt3)
    p2 = pendulum.interval(dt2, dt3)
    it1 = p1.as_timedelta()
    it2 = p2.as_timedelta()

    assert it1.total_seconds() == p1.total_seconds()
    assert it2.total_seconds() == p2.total_seconds()
    assert it1.days == p1.days
    assert it2.days == p2.days
    assert it1.seconds == p1.seconds
    assert it2.seconds == p2.seconds
    assert it1.microseconds == p1.microseconds
    assert it2.microseconds == p2.microseconds


def test_different_timezones_same_time():
    dt1 = pendulum.datetime(2013, 3, 31, 1, 30, tz="Europe/Paris")
    dt2 = pendulum.datetime(2013, 4, 1, 1, 30, tz="Europe/Paris")
    interval = dt2 - dt1

    assert interval.in_words() == "1 day"
    assert interval.in_hours() == 23

    dt1 = pendulum.datetime(2013, 3, 31, 1, 30, tz="Europe/Paris")
    dt2 = pendulum.datetime(2013, 4, 1, 1, 30, tz="America/Toronto")
    interval = dt2 - dt1

    assert interval.in_words() == "1 day 5 hours"
    assert interval.in_hours() == 29



from __future__ import annotations

import pendulum

from pendulum.helpers import local_time


def test_local_time_positive_integer():
    d = pendulum.datetime(2016, 8, 7, 12, 34, 56, 123456)

    t = local_time(d.int_timestamp, 0, d.microsecond)
    assert d.year == t[0]
    assert d.month == t[1]
    assert d.day == t[2]
    assert d.hour == t[3]
    assert d.minute == t[4]
    assert d.second == t[5]
    assert d.microsecond == t[6]


def test_local_time_negative_integer():
    d = pendulum.datetime(1951, 8, 7, 12, 34, 56, 123456)

    t = local_time(d.int_timestamp, 0, d.microsecond)
    assert d.year == t[0]
    assert d.month == t[1]
    assert d.day == t[2]
    assert d.hour == t[3]
    assert d.minute == t[4]
    assert d.second == t[5]
    assert d.microsecond == t[6]



from __future__ import annotations

import pendulum

from tests.conftest import assert_datetime


def test_in_timezone():
    d = pendulum.datetime(2015, 1, 15, 18, 15, 34)
    now = pendulum.datetime(2015, 1, 15, 18, 15, 34)
    assert d.timezone_name == "UTC"
    assert_datetime(d, now.year, now.month, now.day, now.hour, now.minute)

    d = d.in_timezone("Europe/Paris")
    assert d.timezone_name == "Europe/Paris"
    assert_datetime(d, now.year, now.month, now.day, now.hour + 1, now.minute)


def test_in_tz():
    d = pendulum.datetime(2015, 1, 15, 18, 15, 34)
    now = pendulum.datetime(2015, 1, 15, 18, 15, 34)
    assert d.timezone_name == "UTC"
    assert_datetime(d, now.year, now.month, now.day, now.hour, now.minute)

    d = d.in_tz("Europe/Paris")
    assert d.timezone_name == "Europe/Paris"
    assert_datetime(d, now.year, now.month, now.day, now.hour + 1, now.minute)


def test_astimezone():
    d = pendulum.datetime(2015, 1, 15, 18, 15, 34)
    now = pendulum.datetime(2015, 1, 15, 18, 15, 34)
    assert d.timezone_name == "UTC"
    assert_datetime(d, now.year, now.month, now.day, now.hour, now.minute)

    d = d.astimezone(pendulum.timezone("Europe/Paris"))
    assert d.timezone_name == "Europe/Paris"
    assert_datetime(d, now.year, now.month, now.day, now.hour + 1, now.minute)



from __future__ import annotations

from datetime import timedelta

import pytest

import pendulum


def test_add_hours_positive():
    assert pendulum.time(12, 34, 56).add(hours=1).hour == 13


def test_add_hours_zero():
    assert pendulum.time(12, 34, 56).add(hours=0).hour == 12


def test_add_hours_negative():
    assert pendulum.time(12, 34, 56).add(hours=-1).hour == 11


def test_add_minutes_positive():
    assert pendulum.time(12, 34, 56).add(minutes=1).minute == 35


def test_add_minutes_zero():
    assert pendulum.time(12, 34, 56).add(minutes=0).minute == 34


def test_add_minutes_negative():
    assert pendulum.time(12, 34, 56).add(minutes=-1).minute == 33


def test_add_seconds_positive():
    assert pendulum.time(12, 34, 56).add(seconds=1).second == 57


def test_add_seconds_zero():
    assert pendulum.time(12, 34, 56).add(seconds=0).second == 56


def test_add_seconds_negative():
    assert pendulum.time(12, 34, 56).add(seconds=-1).second == 55


def test_add_timedelta():
    delta = timedelta(seconds=45, microseconds=123456)
    d = pendulum.time(3, 12, 15, 654321)

    d = d.add_timedelta(delta)
    assert d.minute == 13
    assert d.second == 0
    assert d.microsecond == 777777

    d = pendulum.time(3, 12, 15, 654321)

    d = d + delta
    assert d.minute == 13
    assert d.second == 0
    assert d.microsecond == 777777


def test_add_timedelta_with_days():
    delta = timedelta(days=3, seconds=45, microseconds=123456)
    d = pendulum.time(3, 12, 15, 654321)

    with pytest.raises(TypeError):
        d.add_timedelta(delta)


def test_addition_invalid_type():
    d = pendulum.time(3, 12, 15, 654321)

    with pytest.raises(TypeError):
        d + 3

    with pytest.raises(TypeError):
        3 + d



from __future__ import annotations

from pendulum.locales.sv.custom import translations as custom_translations


"""
sv locale file.

It has been generated automatically and must not be modified directly.
"""


locale = {
    "plural": lambda n: "one"
    if ((n == n and (n == 1)) and (0 == 0 and (0 == 0)))
    else "other",
    "ordinal": lambda n: "one"
    if (
        ((n % 10) == (n % 10) and (((n % 10) == 1) or ((n % 10) == 2)))
        and (not ((n % 100) == (n % 100) and (((n % 100) == 11) or ((n % 100) == 12))))
    )
    else "other",
    "translations": {
        "days": {
            "abbreviated": {
                0: "mån",
                1: "tis",
                2: "ons",
                3: "tors",
                4: "fre",
                5: "lör",
                6: "sön",
            },
            "narrow": {
                0: "M",
                1: "T",
                2: "O",
                3: "T",
                4: "F",
                5: "L",
                6: "S",
            },
            "short": {
                0: "må",
                1: "ti",
                2: "on",
                3: "to",
                4: "fr",
                5: "lö",
                6: "sö",
            },
            "wide": {
                0: "måndag",
                1: "tisdag",
                2: "onsdag",
                3: "torsdag",
                4: "fredag",
                5: "lördag",
                6: "söndag",
            },
        },
        "months": {
            "abbreviated": {
                1: "jan.",
                2: "feb.",
                3: "mars",
                4: "apr.",
                5: "maj",
                6: "juni",
                7: "juli",
                8: "aug.",
                9: "sep.",
                10: "okt.",
                11: "nov.",
                12: "dec.",
            },
            "narrow": {
                1: "J",
                2: "F",
                3: "M",
                4: "A",
                5: "M",
                6: "J",
                7: "J",
                8: "A",
                9: "S",
                10: "O",
                11: "N",
                12: "D",
            },
            "wide": {
                1: "januari",
                2: "februari",
                3: "mars",
                4: "april",
                5: "maj",
                6: "juni",
                7: "juli",
                8: "augusti",
                9: "september",
                10: "oktober",
                11: "november",
                12: "december",
            },
        },
        "units": {
            "year": {
                "one": "{0} år",
                "other": "{0} år",
            },
            "month": {
                "one": "{0} månad",
                "other": "{0} månader",
            },
            "week": {
                "one": "{0} vecka",
                "other": "{0} veckor",
            },
            "day": {
                "one": "{0} dygn",
                "other": "{0} dygn",
            },
            "hour": {
                "one": "{0} timme",
                "other": "{0} timmar",
            },
            "minute": {
                "one": "{0} minut",
                "other": "{0} minuter",
            },
            "second": {
                "one": "{0} sekund",
                "other": "{0} sekunder",
            },
            "microsecond": {
                "one": "{0} mikrosekund",
                "other": "{0} mikrosekunder",
            },
        },
        "relative": {
            "year": {
                "future": {
                    "other": "om {0} år",
                    "one": "om {0} år",
                },
                "past": {
                    "other": "för {0} år sedan",
                    "one": "för {0} år sedan",
                },
            },
            "month": {
                "future": {
                    "other": "om {0} månader",
                    "one": "om {0} månad",
                },
                "past": {
                    "other": "för {0} månader sedan",
                    "one": "för {0} månad sedan",
                },
            },
            "week": {
                "future": {
                    "other": "om {0} veckor",
                    "one": "om {0} vecka",
                },
                "past": {
                    "other": "för {0} veckor sedan",
                    "one": "för {0} vecka sedan",
                },
            },
            "day": {
                "future": {
                    "other": "om {0} dagar",
                    "one": "om {0} dag",
                },
                "past": {
                    "other": "för {0} dagar sedan",
                    "one": "för {0} dag sedan",
                },
            },
            "hour": {
                "future": {
                    "other": "om {0} timmar",
                    "one": "om {0} timme",
                },
                "past": {
                    "other": "för {0} timmar sedan",
                    "one": "för {0} timme sedan",
                },
            },
            "minute": {
                "future": {
                    "other": "om {0} minuter",
                    "one": "om {0} minut",
                },
                "past": {
                    "other": "för {0} minuter sedan",
                    "one": "för {0} minut sedan",
                },
            },
            "second": {
                "future": {
                    "other": "om {0} sekunder",
                    "one": "om {0} sekund",
                },
                "past": {
                    "other": "för {0} sekunder sedan",
                    "one": "för {0} sekund sedan",
                },
            },
        },
        "day_periods": {
            "midnight": "midnatt",
            "am": "fm",
            "pm": "em",
            "morning1": "på morgonen",
            "morning2": "på förmiddagen",
            "afternoon1": "på eftermiddagen",
            "evening1": "på kvällen",
            "night1": "på natten",
        },
        "week_data": {
            "min_days": 1,
            "first_day": 0,
            "weekend_start": 5,
            "weekend_end": 6,
        },
    },
    "custom": custom_translations,
}



from __future__ import annotations

from pendulum.locales.en_gb.custom import translations as custom_translations


"""
en-gb locale file.

It has been generated automatically and must not be modified directly.
"""


locale = {
    "plural": lambda n: "one"
    if ((n == n and (n == 1)) and (0 == 0 and (0 == 0)))
    else "other",
    "ordinal": lambda n: "few"
    if (
        ((n % 10) == (n % 10) and ((n % 10) == 3))
        and (not ((n % 100) == (n % 100) and ((n % 100) == 13)))
    )
    else "one"
    if (
        ((n % 10) == (n % 10) and ((n % 10) == 1))
        and (not ((n % 100) == (n % 100) and ((n % 100) == 11)))
    )
    else "two"
    if (
        ((n % 10) == (n % 10) and ((n % 10) == 2))
        and (not ((n % 100) == (n % 100) and ((n % 100) == 12)))
    )
    else "other",
    "translations": {
        "days": {
            "abbreviated": {
                0: "Mon",
                1: "Tue",
                2: "Wed",
                3: "Thu",
                4: "Fri",
                5: "Sat",
                6: "Sun",
            },
            "narrow": {
                0: "M",
                1: "T",
                2: "W",
                3: "T",
                4: "F",
                5: "S",
                6: "S",
            },
            "short": {
                0: "Mo",
                1: "Tu",
                2: "We",
                3: "Th",
                4: "Fr",
                5: "Sa",
                6: "Su",
            },
            "wide": {
                0: "Monday",
                1: "Tuesday",
                2: "Wednesday",
                3: "Thursday",
                4: "Friday",
                5: "Saturday",
                6: "Sunday",
            },
        },
        "months": {
            "abbreviated": {
                1: "Jan",
                2: "Feb",
                3: "Mar",
                4: "Apr",
                5: "May",
                6: "Jun",
                7: "Jul",
                8: "Aug",
                9: "Sept",
                10: "Oct",
                11: "Nov",
                12: "Dec",
            },
            "narrow": {
                1: "J",
                2: "F",
                3: "M",
                4: "A",
                5: "M",
                6: "J",
                7: "J",
                8: "A",
                9: "S",
                10: "O",
                11: "N",
                12: "D",
            },
            "wide": {
                1: "January",
                2: "February",
                3: "March",
                4: "April",
                5: "May",
                6: "June",
                7: "July",
                8: "August",
                9: "September",
                10: "October",
                11: "November",
                12: "December",
            },
        },
        "units": {
            "year": {
                "one": "{0} year",
                "other": "{0} years",
            },
            "month": {
                "one": "{0} month",
                "other": "{0} months",
            },
            "week": {
                "one": "{0} week",
                "other": "{0} weeks",
            },
            "day": {
                "one": "{0} day",
                "other": "{0} days",
            },
            "hour": {
                "one": "{0} hour",
                "other": "{0} hours",
            },
            "minute": {
                "one": "{0} minute",
                "other": "{0} minutes",
            },
            "second": {
                "one": "{0} second",
                "other": "{0} seconds",
            },
            "microsecond": {
                "one": "{0} microsecond",
                "other": "{0} microseconds",
            },
        },
        "relative": {
            "year": {
                "future": {
                    "other": "in {0} years",
                    "one": "in {0} year",
                },
                "past": {
                    "other": "{0} years ago",
                    "one": "{0} year ago",
                },
            },
            "month": {
                "future": {
                    "other": "in {0} months",
                    "one": "in {0} month",
                },
                "past": {
                    "other": "{0} months ago",
                    "one": "{0} month ago",
                },
            },
            "week": {
                "future": {
                    "other": "in {0} weeks",
                    "one": "in {0} week",
                },
                "past": {
                    "other": "{0} weeks ago",
                    "one": "{0} week ago",
                },
            },
            "day": {
                "future": {
                    "other": "in {0} days",
                    "one": "in {0} day",
                },
                "past": {
                    "other": "{0} days ago",
                    "one": "{0} day ago",
                },
            },
            "hour": {
                "future": {
                    "other": "in {0} hours",
                    "one": "in {0} hour",
                },
                "past": {
                    "other": "{0} hours ago",
                    "one": "{0} hour ago",
                },
            },
            "minute": {
                "future": {
                    "other": "in {0} minutes",
                    "one": "in {0} minute",
                },
                "past": {
                    "other": "{0} minutes ago",
                    "one": "{0} minute ago",
                },
            },
            "second": {
                "future": {
                    "other": "in {0} seconds",
                    "one": "in {0} second",
                },
                "past": {
                    "other": "{0} seconds ago",
                    "one": "{0} second ago",
                },
            },
        },
        "day_periods": {
            "midnight": "midnight",
            "am": "am",
            "noon": "noon",
            "pm": "pm",
            "morning1": "in the morning",
            "afternoon1": "in the afternoon",
            "evening1": "in the evening",
            "night1": "at night",
        },
        "week_data": {
            "min_days": 4,
            "first_day": 0,
            "weekend_start": 5,
            "weekend_end": 6,
        },
    },
    "custom": custom_translations,
}



from __future__ import annotations

import pytest

import pendulum

from pendulum.formatting import Formatter
from pendulum.locales.locale import Locale


@pytest.fixture(autouse=True)
def setup():
    Locale._cache["dummy"] = {}

    yield

    del Locale._cache["dummy"]


def test_year_tokens():
    d = pendulum.datetime(2009, 1, 14, 15, 25, 50, 123456)
    f = Formatter()

    assert f.format(d, "YYYY") == "2009"
    assert f.format(d, "YY") == "09"
    assert f.format(d, "Y") == "2009"


def test_quarter_tokens():
    f = Formatter()
    d = pendulum.datetime(1985, 1, 4)
    assert f.format(d, "Q") == "1"

    d = pendulum.datetime(2029, 8, 1)
    assert f.format(d, "Q") == "3"

    d = pendulum.datetime(1985, 1, 4)
    assert f.format(d, "Qo") == "1st"

    d = pendulum.datetime(2029, 8, 1)
    assert f.format(d, "Qo") == "3rd"

    d = pendulum.datetime(1985, 1, 4)
    assert f.format(d, "Qo", locale="fr") == "1er"

    d = pendulum.datetime(2029, 8, 1)
    assert f.format(d, "Qo", locale="fr") == "3e"


def test_month_tokens():
    f = Formatter()
    d = pendulum.datetime(2016, 3, 24)
    assert f.format(d, "MM") == "03"
    assert f.format(d, "M") == "3"

    assert f.format(d, "MMM") == "Mar"
    assert f.format(d, "MMMM") == "March"
    assert f.format(d, "Mo") == "3rd"

    assert f.format(d, "MMM", locale="fr") == "mars"
    assert f.format(d, "MMMM", locale="fr") == "mars"
    assert f.format(d, "Mo", locale="fr") == "3e"


def test_day_tokens():
    f = Formatter()
    d = pendulum.datetime(2016, 3, 7)
    assert f.format(d, "DD") == "07"
    assert f.format(d, "D") == "7"

    assert f.format(d, "Do") == "7th"
    assert f.format(d.first_of("month"), "Do") == "1st"

    assert f.format(d, "Do", locale="fr") == "7e"
    assert f.format(d.first_of("month"), "Do", locale="fr") == "1er"


def test_day_of_year():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28)
    assert f.format(d, "DDDD") == "241"
    assert f.format(d, "DDD") == "241"
    assert f.format(d.start_of("year"), "DDDD") == "001"
    assert f.format(d.start_of("year"), "DDD") == "1"

    assert f.format(d, "DDDo") == "241st"
    assert f.format(d.add(days=3), "DDDo") == "244th"

    assert f.format(d, "DDDo", locale="fr") == "241e"
    assert f.format(d.add(days=3), "DDDo", locale="fr") == "244e"


def test_week_of_year():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28)

    assert f.format(d, "wo") == "34th"


def test_day_of_week():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28)
    assert f.format(d, "d") == "0"

    assert f.format(d, "dd") == "Su"
    assert f.format(d, "ddd") == "Sun"
    assert f.format(d, "dddd") == "Sunday"

    assert f.format(d, "dd", locale="fr") == "di"
    assert f.format(d, "ddd", locale="fr") == "dim."
    assert f.format(d, "dddd", locale="fr") == "dimanche"

    assert f.format(d, "do") == "0th"


def test_localized_day_of_week():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28)
    assert f.format(d, "e") == "0"
    assert f.format(d, "e", locale="en-gb") == "6"
    assert f.format(d.add(days=2), "e") == "2"
    assert f.format(d.add(days=2), "e", locale="en-gb") == "1"

    assert f.format(d, "eo") == "1st"
    assert f.format(d, "eo", locale="en-gb") == "7th"
    assert f.format(d.add(days=2), "eo") == "3rd"
    assert f.format(d.add(days=2), "eo", locale="en-gb") == "2nd"


def test_day_of_iso_week():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28)
    assert f.format(d, "E") == "7"


def test_am_pm():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28, 23)
    assert f.format(d, "A") == "PM"
    assert f.format(d.set(hour=11), "A") == "AM"


def test_hour():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28, 7)
    assert f.format(d, "H") == "7"
    assert f.format(d, "HH") == "07"

    d = pendulum.datetime(2016, 8, 28, 0)
    assert f.format(d, "h") == "12"
    assert f.format(d, "hh") == "12"


def test_minute():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28, 7, 3)
    assert f.format(d, "m") == "3"
    assert f.format(d, "mm") == "03"


def test_second():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28, 7, 3, 6)
    assert f.format(d, "s") == "6"
    assert f.format(d, "ss") == "06"


def test_fractional_second():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28, 7, 3, 6, 123456)
    assert f.format(d, "S") == "1"
    assert f.format(d, "SS") == "12"
    assert f.format(d, "SSS") == "123"
    assert f.format(d, "SSSS") == "1234"
    assert f.format(d, "SSSSS") == "12345"
    assert f.format(d, "SSSSSS") == "123456"

    d = pendulum.datetime(2016, 8, 28, 7, 3, 6, 0)
    assert f.format(d, "S") == "0"
    assert f.format(d, "SS") == "00"
    assert f.format(d, "SSS") == "000"
    assert f.format(d, "SSSS") == "0000"
    assert f.format(d, "SSSSS") == "00000"
    assert f.format(d, "SSSSSS") == "000000"

    d = pendulum.datetime(2016, 8, 28, 7, 3, 6, 123)
    assert f.format(d, "S") == "0"
    assert f.format(d, "SS") == "00"
    assert f.format(d, "SSS") == "000"
    assert f.format(d, "SSSS") == "0001"
    assert f.format(d, "SSSSS") == "00012"
    assert f.format(d, "SSSSSS") == "000123"


def test_timezone():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28, 7, 3, 6, 123456, tz="Europe/Paris")
    assert f.format(d, "zz") == "CEST"
    assert f.format(d, "z") == "Europe/Paris"

    d = pendulum.datetime(2016, 1, 28, 7, 3, 6, 123456, tz="Europe/Paris")
    assert f.format(d, "zz") == "CET"
    assert f.format(d, "z") == "Europe/Paris"


def test_timezone_offset():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28, 7, 3, 6, 123456, tz="Europe/Paris")
    assert f.format(d, "ZZ") == "+0200"
    assert f.format(d, "Z") == "+02:00"

    d = pendulum.datetime(2016, 1, 28, 7, 3, 6, 123456, tz="Europe/Paris")
    assert f.format(d, "ZZ") == "+0100"
    assert f.format(d, "Z") == "+01:00"

    d = pendulum.datetime(2016, 1, 28, 7, 3, 6, 123456, tz="America/Guayaquil")
    assert f.format(d, "ZZ") == "-0500"
    assert f.format(d, "Z") == "-05:00"


def test_timestamp():
    f = Formatter()
    d = pendulum.datetime(1970, 1, 1)
    assert f.format(d, "X") == "0"
    assert f.format(d.add(days=1), "X") == "86400"


def test_timestamp_milliseconds():
    f = Formatter()
    d = pendulum.datetime(1970, 1, 1)
    assert f.format(d, "x") == "0"
    assert f.format(d.add(days=1), "x") == "86400000"
    assert f.format(d.add(days=1, microseconds=129123), "x") == "86400129"


def test_date_formats():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28, 7, 3, 6, 123456)
    assert f.format(d, "LT") == "7:03 AM"
    assert f.format(d, "LTS") == "7:03:06 AM"
    assert f.format(d, "L") == "08/28/2016"
    assert f.format(d, "LL") == "August 28, 2016"
    assert f.format(d, "LLL") == "August 28, 2016 7:03 AM"
    assert f.format(d, "LLLL") == "Sunday, August 28, 2016 7:03 AM"

    assert f.format(d, "LT", locale="fr") == "07:03"
    assert f.format(d, "LTS", locale="fr") == "07:03:06"
    assert f.format(d, "L", locale="fr") == "28/08/2016"
    assert f.format(d, "LL", locale="fr") == "28 août 2016"
    assert f.format(d, "LLL", locale="fr") == "28 août 2016 07:03"
    assert f.format(d, "LLLL", locale="fr") == "dimanche 28 août 2016 07:03"


def test_escape():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28)
    assert f.format(d, r"[YYYY] YYYY \[YYYY\]") == "YYYY 2016 [2016]"
    assert f.format(d, r"\D D \\D") == "D 28 \\28"


def test_date_formats_missing():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28, 7, 3, 6, 123456)

    assert f.format(d, "LT", locale="dummy") == "7:03 AM"
    assert f.format(d, "LTS", locale="dummy") == "7:03:06 AM"
    assert f.format(d, "L", locale="dummy") == "08/28/2016"
    assert f.format(d, "LL", locale="dummy") == "August 28, 2016"
    assert f.format(d, "LLL", locale="dummy") == "August 28, 2016 7:03 AM"
    assert f.format(d, "LLLL", locale="dummy") == "Sunday, August 28, 2016 7:03 AM"


def test_unknown_token():
    f = Formatter()
    d = pendulum.datetime(2016, 8, 28, 7, 3, 6, 123456)

    assert f.format(d, "J") == "J"



"""
he custom locale file.
"""
from __future__ import annotations


translations = {
    "units": {"few_second": "כמה שניות"},
    # Relative time
    "ago": "לפני {0}",
    "from_now": "תוך {0}",
    "after": "בעוד {0}",
    "before": "{0} קודם",
    # Ordinals
    "ordinal": {"other": "º"},
    # Date formats
    "date_formats": {
        "LTS": "H:mm:ss",
        "LT": "H:mm",
        "LLLL": "dddd, D [ב] MMMM [ב] YYYY H:mm",
        "LLL": "D [ב] MMMM [ב] YYYY H:mm",
        "LL": "D [ב] MMMM [ב] YYYY",
        "L": "DD/MM/YYYY",
    },
}



from __future__ import annotations

import pendulum

from tests.conftest import assert_date


def test_fluid_year_setter():
    d = pendulum.Date(2016, 10, 20)
    new = d.set(year=1995)

    assert_date(new, 1995, 10, 20)
    assert new.year == 1995


def test_fluid_month_setter():
    d = pendulum.Date(2016, 7, 2)
    new = d.set(month=11)

    assert new.month == 11
    assert d.month == 7


def test_fluid_day_setter():
    d = pendulum.Date(2016, 7, 2)
    new = d.set(day=9)

    assert new.day == 9
    assert d.day == 2



from __future__ import annotations

import pendulum


locale = "ko"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "1초 전"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "2초 전"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "1분 전"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "2분 전"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "1시간 전"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "2시간 전"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "1일 전"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "2일 전"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "1주 전"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "2주 전"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "1개월 전"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "2개월 전"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "1년 전"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "2년 전"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "1초 후"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "1초 후"
    assert d2.diff_for_humans(d, locale=locale) == "1초 전"

    assert d.diff_for_humans(d2, True, locale=locale) == "1초"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "2초"



from __future__ import annotations

import pickle

from datetime import timedelta

import pendulum


def test_pickle():
    dt1 = pendulum.datetime(2016, 11, 18)
    dt2 = pendulum.datetime(2016, 11, 20)

    p = pendulum.interval(dt1, dt2)
    s = pickle.dumps(p)
    p2 = pickle.loads(s)

    assert p.start == p2.start
    assert p.end == p2.end
    assert p.invert == p2.invert

    p = pendulum.interval(dt2, dt1)
    s = pickle.dumps(p)
    p2 = pickle.loads(s)

    assert p.start == p2.start
    assert p.end == p2.end
    assert p.invert == p2.invert

    p = pendulum.interval(dt2, dt1, True)
    s = pickle.dumps(p)
    p2 = pickle.loads(s)

    assert p.start == p2.start
    assert p.end == p2.end
    assert p.invert == p2.invert


def test_comparison_to_timedelta():
    dt1 = pendulum.datetime(2016, 11, 18)
    dt2 = pendulum.datetime(2016, 11, 20)

    interval = dt2 - dt1

    assert interval < timedelta(days=4)


def test_equality_to_timedelta():
    dt1 = pendulum.datetime(2016, 11, 18)
    dt2 = pendulum.datetime(2016, 11, 20)

    interval = dt2 - dt1

    assert interval == timedelta(days=2)


def test_inequality():
    dt1 = pendulum.datetime(2016, 11, 18)
    dt2 = pendulum.datetime(2016, 11, 20)
    dt3 = pendulum.datetime(2016, 11, 22)

    interval1 = dt2 - dt1
    interval2 = dt3 - dt2
    interval3 = dt3 - dt1

    assert interval1 != interval2
    assert interval1 != interval3



from __future__ import annotations

from datetime import time
from datetime import timedelta

import pytest
import pytz

import pendulum

from pendulum import Time
from tests.conftest import assert_duration


def test_sub_hours_positive():
    assert Time(0, 0, 0).subtract(hours=1).hour == 23


def test_sub_hours_zero():
    assert Time(0, 0, 0).subtract(hours=0).hour == 0


def test_sub_hours_negative():
    assert Time(0, 0, 0).subtract(hours=-1).hour == 1


def test_sub_minutes_positive():
    assert Time(0, 0, 0).subtract(minutes=1).minute == 59


def test_sub_minutes_zero():
    assert Time(0, 0, 0).subtract(minutes=0).minute == 0


def test_sub_minutes_negative():
    assert Time(0, 0, 0).subtract(minutes=-1).minute == 1


def test_sub_seconds_positive():
    assert Time(0, 0, 0).subtract(seconds=1).second == 59


def test_sub_seconds_zero():
    assert Time(0, 0, 0).subtract(seconds=0).second == 0


def test_sub_seconds_negative():
    assert Time(0, 0, 0).subtract(seconds=-1).second == 1


def test_subtract_timedelta():
    delta = timedelta(seconds=16, microseconds=654321)
    d = Time(3, 12, 15, 777777)

    d = d.subtract_timedelta(delta)
    assert d.minute == 11
    assert d.second == 59
    assert d.microsecond == 123456

    d = Time(3, 12, 15, 777777)

    d = d - delta
    assert d.minute == 11
    assert d.second == 59
    assert d.microsecond == 123456


def test_add_timedelta_with_days():
    delta = timedelta(days=3, seconds=45, microseconds=123456)
    d = Time(3, 12, 15, 654321)

    with pytest.raises(TypeError):
        d.subtract_timedelta(delta)


def test_subtract_invalid_type():
    d = Time(0, 0, 0)

    with pytest.raises(TypeError):
        d - "ab"

    with pytest.raises(TypeError):
        "ab" - d


def test_subtract_time():
    t = Time(12, 34, 56)
    t1 = Time(1, 1, 1)
    t2 = time(1, 1, 1)
    t3 = time(1, 1, 1, tzinfo=pytz.timezone("Europe/Paris"))

    diff = t - t1
    assert isinstance(diff, pendulum.Duration)
    assert_duration(diff, 0, hours=11, minutes=33, seconds=55)

    diff = t1 - t
    assert isinstance(diff, pendulum.Duration)
    assert_duration(diff, 0, hours=-11, minutes=-33, seconds=-55)

    diff = t - t2
    assert isinstance(diff, pendulum.Duration)
    assert_duration(diff, 0, hours=11, minutes=33, seconds=55)

    diff = t2 - t
    assert isinstance(diff, pendulum.Duration)
    assert_duration(diff, 0, hours=-11, minutes=-33, seconds=-55)

    with pytest.raises(TypeError):
        t - t3

    with pytest.raises(TypeError):
        t3 - t



from __future__ import annotations

import pendulum


def test_dst_add():
    start = pendulum.datetime(2017, 3, 7, tz="America/Toronto")
    end = start.add(days=6)
    interval = end - start
    new_end = start + interval

    assert new_end == end


def test_dst_add_non_variable_units():
    start = pendulum.datetime(2013, 3, 31, 1, 30, tz="Europe/Paris")
    end = start.add(hours=1)
    interval = end - start
    new_end = start + interval

    assert new_end == end


def test_dst_subtract():
    start = pendulum.datetime(2017, 3, 7, tz="America/Toronto")
    end = start.add(days=6)
    interval = end - start
    new_start = end - interval

    assert new_start == start


def test_naive_subtract():
    start = pendulum.naive(2013, 3, 31, 1, 30)
    end = start.add(hours=1)
    interval = end - start
    new_end = start + interval

    assert new_end == end


def test_negative_difference_subtract():
    start = pendulum.datetime(2018, 5, 28, 12, 34, 56, 123456)
    end = pendulum.datetime(2018, 1, 1)

    interval = end - start
    new_end = start + interval

    assert new_end == end



from __future__ import annotations

import pendulum


def test_intervals_with_same_duration_and_different_dates():
    day1 = pendulum.DateTime(2018, 1, 1)
    day2 = pendulum.DateTime(2018, 1, 2)
    day3 = pendulum.DateTime(2018, 1, 2)

    interval1 = day2 - day1
    interval2 = day3 - day2

    assert interval1 != interval2
    assert len({interval1, interval2}) == 2


def test_intervals_with_same_dates():
    interval1 = pendulum.DateTime(2018, 1, 2) - pendulum.DateTime(2018, 1, 1)
    interval2 = pendulum.DateTime(2018, 1, 2) - pendulum.DateTime(2018, 1, 1)

    assert interval1 == interval2
    assert len({interval1, interval2}) == 1



from __future__ import annotations

from pendulum.locales.ua.custom import translations as custom_translations


"""
ua locale file.

It has been generated automatically and must not be modified directly.
"""


locale = {
    "plural": lambda n: "few"
    if (
        (
            (0 == 0 and (0 == 0))
            and ((n % 10) == (n % 10) and ((n % 10) >= 2 and (n % 10) <= 4))
        )
        and (not ((n % 100) == (n % 100) and ((n % 100) >= 12 and (n % 100) <= 14)))
    )
    else "many"
    if (
        (
            ((0 == 0 and (0 == 0)) and ((n % 10) == (n % 10) and ((n % 10) == 0)))
            or (
                (0 == 0 and (0 == 0))
                and ((n % 10) == (n % 10) and ((n % 10) >= 5 and (n % 10) <= 9))
            )
        )
        or (
            (0 == 0 and (0 == 0))
            and ((n % 100) == (n % 100) and ((n % 100) >= 11 and (n % 100) <= 14))
        )
    )
    else "one"
    if (
        ((0 == 0 and (0 == 0)) and ((n % 10) == (n % 10) and ((n % 10) == 1)))
        and (not ((n % 100) == (n % 100) and ((n % 100) == 11)))
    )
    else "other",
    "ordinal": lambda n: "other",
    "translations": {
        "days": {
            "abbreviated": {
                0: "пн",
                1: "вт",
                2: "ср",
                3: "чт",
                4: "пт",
                5: "сб",
                6: "нд",
            },
            "narrow": {0: "пн", 1: "вт", 2: "ср", 3: "чт", 4: "пт", 5: "сб", 6: "нд"},
            "short": {0: "пн", 1: "вт", 2: "ср", 3: "чт", 4: "пт", 5: "сб", 6: "нд"},
            "wide": {
                0: "понеділок",
                1: "вівторок",
                2: "середа",
                3: "четвер",
                4: "п'ятниця",
                5: "субота",
                6: "неділя",
            },
        },
        "months": {
            "abbreviated": {
                1: "січ.",
                2: "лют.",
                3: "бер.",
                4: "квіт.",
                5: "трав.",
                6: "черв.",
                7: "лип.",
                8: "серп.",
                9: "вер.",
                10: "жовт.",
                11: "лист.",
                12: "груд.",
            },
            "narrow": {
                1: "С",
                2: "Л",
                3: "Б",
                4: "К",
                5: "Т",
                6: "Ч",
                7: "Л",
                8: "С",
                9: "В",
                10: "Ж",
                11: "Л",
                12: "Г",
            },
            "wide": {
                1: "січня",
                2: "лютого",
                3: "березня",
                4: "квітня",
                5: "травня",
                6: "червня",
                7: "липня",
                8: "серпня",
                9: "вересня",
                10: "жовтня",
                11: "листопада",
                12: "грудня",
            },
        },
        "units": {
            "year": {
                "one": "{0} рік",
                "few": "{0} роки",
                "many": "{0} років",
                "other": "{0} роки",
            },
            "month": {
                "one": "{0} місяць",
                "few": "{0} місяця",
                "many": "{0} місяців",
                "other": "{0} місяці",
            },
            "week": {
                "one": "{0} тиждень",
                "few": "{0} тижня",
                "many": "{0} тижнів",
                "other": "{0} тижні",
            },
            "day": {
                "one": "{0} день",
                "few": "{0} дні",
                "many": "{0} днів",
                "other": "{0} дні",
            },
            "hour": {
                "one": "{0} година",
                "few": "{0} години",
                "many": "{0} годин",
                "other": "{0} години",
            },
            "minute": {
                "one": "{0} хвилина",
                "few": "{0} хвилини",
                "many": "{0} хвилин",
                "other": "{0} хвилини",
            },
            "second": {
                "one": "{0} секунда",
                "few": "{0} секунди",
                "many": "{0} секунд",
                "other": "{0} секунди",
            },
            "microsecond": {
                "one": "{0} мікросекунда",
                "few": "{0} мікросекунди",
                "many": "{0} мікросекунд",
                "other": "{0} мікросекунд",
            },
        },
        "relative": {
            "year": {
                "future": {
                    "other": "за {0} роки",
                    "one": "за {0} рік",
                    "few": "за {0} роки",
                    "many": "за {0} років",
                },
                "past": {
                    "other": "{0} роки тому",
                    "one": "{0} рік тому",
                    "few": "{0} роки тому",
                    "many": "{0} років тому",
                },
            },
            "month": {
                "future": {
                    "other": "за {0} місяці",
                    "one": "за {0} місяць",
                    "few": "за {0} місяця",
                    "many": "за {0} місяців",
                },
                "past": {
                    "other": "{0} місяці тому",
                    "one": "{0} місяц тому",
                    "few": "{0} місяця тому",
                    "many": "{0} місяців тому",
                },
            },
            "week": {
                "future": {
                    "other": "за {0} тижні",
                    "one": "за {0} тиждень",
                    "few": "за {0} тижня",
                    "many": "за {0} тижднів",
                },
                "past": {
                    "other": "{0} тижні тому",
                    "one": "{0} тиждень тому",
                    "few": "{0} тижня тому",
                    "many": "{0} тижнів тому",
                },
            },
            "day": {
                "future": {
                    "other": "за {0} дні",
                    "one": "за {0} день",
                    "few": "за {0} дні",
                    "many": "за {0} днів",
                },
                "past": {
                    "other": "{0} дні тому",
                    "one": "{0} день тому",
                    "few": "{0} дні тому",
                    "many": "{0} днів тому",
                },
            },
            "hour": {
                "future": {
                    "other": "за {0} години",
                    "one": "за {0} година",
                    "few": "за {0} години",
                    "many": "за {0} годин",
                },
                "past": {
                    "other": "{0} години тому",
                    "one": "{0} година тому",
                    "few": "{0} години тому",
                    "many": "{0} годин тому",
                },
            },
            "minute": {
                "future": {
                    "other": "за {0} хвилини",
                    "one": "за {0} хвилина",
                    "few": "за {0} хвилини",
                    "many": "за {0} хвилин",
                },
                "past": {
                    "other": "{0} хвилини тому",
                    "one": "{0} хвилина тому",
                    "few": "{0} хвилини тому",
                    "many": "{0} хвилин тому",
                },
            },
            "second": {
                "future": {
                    "other": "за {0} секунди",
                    "one": "за {0} секунду",
                    "few": "за {0} секунди",
                    "many": "за {0} секунд",
                },
                "past": {
                    "other": "{0} секунди тому",
                    "one": "{0} секунду тому",
                    "few": "{0} секунди тому",
                    "many": "{0} секунд тому",
                },
            },
        },
        "day_periods": {
            "midnight": "опівночі",
            "am": "AM",
            "noon": "полудень",
            "pm": "PM",
            "morning1": "ранку",
            "morning2": "до півдня",
            "afternoon1": "дня",
            "afternoon2": "пополуднє",
            "evening1": "ввечері",
            "evening2": "увечері",
            "night1": "в ніч",
        },
        "week_data": {
            "min_days": 1,
            "first_day": 0,
            "weekend_start": 5,
            "weekend_end": 6,
        },
    },
    "custom": custom_translations,
}



from __future__ import annotations

import pendulum


locale = "bg"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "преди 1 секунда"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "преди 2 секунди"

    d = pendulum.now().subtract(seconds=5)
    assert d.diff_for_humans(locale=locale) == "преди 5 секунди"

    d = pendulum.now().subtract(seconds=21)
    assert d.diff_for_humans(locale=locale) == "преди 21 секунди"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "преди 1 минута"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "преди 2 минути"

    d = pendulum.now().subtract(minutes=5)
    assert d.diff_for_humans(locale=locale) == "преди 5 минути"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "преди 1 час"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "преди 2 часа"

    d = pendulum.now().subtract(hours=5)
    assert d.diff_for_humans(locale=locale) == "преди 5 часа"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "преди 1 ден"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "преди 2 дни"

    d = pendulum.now().subtract(days=5)
    assert d.diff_for_humans(locale=locale) == "преди 5 дни"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "преди 1 седмица"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "преди 2 седмици"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "преди 1 месец"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "преди 2 месеца"

    d = pendulum.now().subtract(months=5)
    assert d.diff_for_humans(locale=locale) == "преди 5 месеца"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "преди 1 година"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "преди 2 години"

    d = pendulum.now().subtract(years=5)
    assert d.diff_for_humans(locale=locale) == "преди 5 години"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "след 1 секунда"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "след 1 секунда"
    assert d2.diff_for_humans(d, locale=locale) == "преди 1 секунда"

    assert d.diff_for_humans(d2, True, locale=locale) == "1 секунда"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "2 секунди"



from __future__ import annotations

import pytest

import pendulum


def test_to_string():
    d = pendulum.datetime(1975, 12, 25, 0, 0, 0, 0, tz="local")
    assert str(d) == "1975-12-25 00:00:00-05:00"
    d = pendulum.datetime(1975, 12, 25, 0, 0, 0, 123456, tz="local")
    assert str(d) == "1975-12-25 00:00:00.123456-05:00"


def test_to_date_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16)

    assert d.to_date_string() == "1975-12-25"


def test_to_formatted_date_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16)

    assert d.to_formatted_date_string() == "Dec 25, 1975"


def test_to_timestring():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16)

    assert d.to_time_string() == "14:15:16"


def test_to_atom_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.to_atom_string() == "1975-12-25T14:15:16-05:00"


def test_to_cookie_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.to_cookie_string() == "Thursday, 25-Dec-1975 14:15:16 EST"


def test_to_iso8601_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.to_iso8601_string() == "1975-12-25T14:15:16-05:00"


def test_to_iso8601_string_utc():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16)
    assert d.to_iso8601_string() == "1975-12-25T14:15:16Z"


def test_to_iso8601_extended_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, 123456, tz="local")
    assert d.to_iso8601_string() == "1975-12-25T14:15:16.123456-05:00"


def test_to_rfc822_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.to_rfc822_string() == "Thu, 25 Dec 75 14:15:16 -0500"


def test_to_rfc850_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.to_rfc850_string() == "Thursday, 25-Dec-75 14:15:16 EST"


def test_to_rfc1036_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.to_rfc1036_string() == "Thu, 25 Dec 75 14:15:16 -0500"


def test_to_rfc1123_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.to_rfc1123_string() == "Thu, 25 Dec 1975 14:15:16 -0500"


def test_to_rfc2822_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.to_rfc2822_string() == "Thu, 25 Dec 1975 14:15:16 -0500"


def test_to_rfc3339_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.to_rfc3339_string() == "1975-12-25T14:15:16-05:00"


def test_to_rfc3339_extended_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, 123456, tz="local")
    assert d.to_rfc3339_string() == "1975-12-25T14:15:16.123456-05:00"


def test_to_rss_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.to_rss_string() == "Thu, 25 Dec 1975 14:15:16 -0500"


def test_to_w3c_string():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.to_w3c_string() == "1975-12-25T14:15:16-05:00"


def test_to_string_invalid():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")

    with pytest.raises(ValueError):
        d._to_string("invalid")


def test_repr():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    expected = f"DateTime(1975, 12, 25, 14, 15, 16, tzinfo={d.tzinfo!r})"
    assert repr(d) == expected

    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, 123456, tz="local")
    expected = f"DateTime(1975, 12, 25, 14, 15, 16, 123456, tzinfo={d.tzinfo!r})"
    assert repr(d) == expected


def test_format_with_locale():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    expected = "jeudi 25e jour de décembre 1975 02:15:16 PM -05:00"
    assert d.format("dddd Do [jour de] MMMM YYYY hh:mm:ss A Z", locale="fr") == expected


def test_strftime():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.strftime("%d") == "25"


def test_for_json():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="local")
    assert d.for_json() == "1975-12-25T14:15:16-05:00"


def test_format():
    d = pendulum.datetime(1975, 12, 25, 14, 15, 16, tz="Europe/Paris")
    assert f"{d}" == "1975-12-25 14:15:16+01:00"
    assert f"{d:YYYY}" == "1975"
    assert f"{d:%Y}" == "1975"
    assert f"{d:%H:%M %d.%m.%Y}" == "14:15 25.12.1975"



from __future__ import annotations

from time import sleep
from typing import TYPE_CHECKING

import pytest

import pendulum

from pendulum.utils._compat import PYPY


if TYPE_CHECKING:
    from collections.abc import Generator


@pytest.fixture(autouse=True)
def setup() -> Generator[None, None, None]:
    pendulum.travel_back()

    yield

    pendulum.travel_back()


@pytest.mark.skipif(PYPY, reason="Time travelling not available on PyPy")
def test_travel() -> None:
    now = pendulum.now()

    pendulum.travel(minutes=5)

    assert pendulum.now().diff_for_humans(now) == "5 minutes after"


@pytest.mark.skipif(PYPY, reason="Time travelling not available on PyPy")
def test_travel_with_frozen_time() -> None:
    pendulum.travel(minutes=5, freeze=True)

    now = pendulum.now()

    sleep(0.01)

    assert now == pendulum.now()


@pytest.mark.skipif(PYPY, reason="Time travelling not available on PyPy")
def test_travel_to() -> None:
    dt = pendulum.datetime(2022, 1, 19, tz="local")

    pendulum.travel_to(dt)

    assert pendulum.now().date() == dt.date()


@pytest.mark.skipif(PYPY, reason="Time travelling not available on PyPy")
def test_freeze() -> None:
    pendulum.freeze()

    pendulum.travel(minutes=5)

    assert pendulum.now() == pendulum.now()

    pendulum.travel_back()

    pendulum.travel(minutes=5)

    now = pendulum.now()

    sleep(0.01)

    assert now != pendulum.now()

    pendulum.freeze()

    assert pendulum.now() == pendulum.now()

    pendulum.travel_back()

    with pendulum.freeze():
        assert pendulum.now() == pendulum.now()

    now = pendulum.now()

    sleep(0.01)

    assert now != pendulum.now()



from __future__ import annotations

import pendulum


locale = "fo"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "1 sekund síðan"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "2 sekund síðan"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "1 minutt síðan"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "2 minuttir síðan"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "1 tími síðan"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "2 tímar síðan"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "1 dagur síðan"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "2 dagar síðan"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "1 vika síðan"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "2 vikur síðan"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "1 mánað síðan"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "2 mánaðir síðan"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "1 ár síðan"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "2 ár síðan"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "um 1 sekund"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "1 sekund aftaná"
    assert d2.diff_for_humans(d, locale=locale) == "1 sekund áðrenn"

    assert d.diff_for_humans(d2, True, locale=locale) == "1 sekund"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "2 sekundir"



from __future__ import annotations

from pendulum import Date
from tests.conftest import assert_date


def test_construct():
    d = Date(2016, 10, 20)

    assert_date(d, 2016, 10, 20)


def test_today():
    d = Date.today()

    assert isinstance(d, Date)



"""
bg custom locale file.
"""
from __future__ import annotations


translations = {
    # Relative time
    "ago": "преди {}",
    "from_now": "след {}",
    "after": "след {0}",
    "before": "преди {0}",
    # Date formats
    "date_formats": {
        "LTS": "HH:mm:ss",
        "LT": "HH:mm",
        "L": "DD.MM.YYYY",
        "LL": "D MMMM YYYY г.",
        "LLL": "D MMMM YYYY г., HH:mm",
        "LLLL": "dddd, D MMMM YYYY г., HH:mm",
    },
}



from __future__ import annotations

import pendulum


locale = "cs"


def test_diff_for_humans():
    with pendulum.travel_to(pendulum.datetime(2016, 8, 29), freeze=True):
        diff_for_humans()


def diff_for_humans():
    d = pendulum.now().subtract(seconds=1)
    assert d.diff_for_humans(locale=locale) == "pár vteřin zpět"

    d = pendulum.now().subtract(seconds=2)
    assert d.diff_for_humans(locale=locale) == "pár vteřin zpět"

    d = pendulum.now().subtract(seconds=20)
    assert d.diff_for_humans(locale=locale) == "před 20 sekundami"

    d = pendulum.now().subtract(minutes=1)
    assert d.diff_for_humans(locale=locale) == "před 1 minutou"

    d = pendulum.now().subtract(minutes=2)
    assert d.diff_for_humans(locale=locale) == "před 2 minutami"

    d = pendulum.now().subtract(minutes=5)
    assert d.diff_for_humans(locale=locale) == "před 5 minutami"

    d = pendulum.now().subtract(hours=1)
    assert d.diff_for_humans(locale=locale) == "před 1 hodinou"

    d = pendulum.now().subtract(hours=2)
    assert d.diff_for_humans(locale=locale) == "před 2 hodinami"

    d = pendulum.now().subtract(hours=5)
    assert d.diff_for_humans(locale=locale) == "před 5 hodinami"

    d = pendulum.now().subtract(days=1)
    assert d.diff_for_humans(locale=locale) == "před 1 dnem"

    d = pendulum.now().subtract(days=2)
    assert d.diff_for_humans(locale=locale) == "před 2 dny"

    d = pendulum.now().subtract(weeks=1)
    assert d.diff_for_humans(locale=locale) == "před 1 týdnem"

    d = pendulum.now().subtract(weeks=2)
    assert d.diff_for_humans(locale=locale) == "před 2 týdny"

    d = pendulum.now().subtract(months=1)
    assert d.diff_for_humans(locale=locale) == "před 1 měsícem"

    d = pendulum.now().subtract(months=2)
    assert d.diff_for_humans(locale=locale) == "před 2 měsíci"

    d = pendulum.now().subtract(months=5)
    assert d.diff_for_humans(locale=locale) == "před 5 měsíci"

    d = pendulum.now().subtract(years=1)
    assert d.diff_for_humans(locale=locale) == "před 1 rokem"

    d = pendulum.now().subtract(years=2)
    assert d.diff_for_humans(locale=locale) == "před 2 lety"

    d = pendulum.now().subtract(years=5)
    assert d.diff_for_humans(locale=locale) == "před 5 lety"

    d = pendulum.now().add(seconds=1)
    assert d.diff_for_humans(locale=locale) == "za pár vteřin"

    d = pendulum.now().add(seconds=1)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "pár vteřin po"
    assert d2.diff_for_humans(d, locale=locale) == "pár vteřin zpět"

    assert d.diff_for_humans(d2, True, locale=locale) == "pár vteřin"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "pár vteřin"

    d = pendulum.now().add(seconds=20)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, locale=locale) == "20 sekund po"
    assert d2.diff_for_humans(d, locale=locale) == "20 sekund zpět"

    d = pendulum.now().add(seconds=10)
    d2 = pendulum.now()
    assert d.diff_for_humans(d2, True, locale=locale) == "pár vteřin"
    assert d2.diff_for_humans(d.add(seconds=1), True, locale=locale) == "11 sekund"


def test_format():
    d = pendulum.datetime(2016, 8, 29, 7, 3, 6, 123456)
    assert d.format("dddd", locale=locale) == "pondělí"
    assert d.format("ddd", locale=locale) == "po"
    assert d.format("MMMM", locale=locale) == "srpna"
    assert d.format("MMM", locale=locale) == "srp"
    assert d.format("A", locale=locale) == "dop."
    assert d.format("Qo", locale=locale) == "3."
    assert d.format("Mo", locale=locale) == "8."
    assert d.format("Do", locale=locale) == "29."

    assert d.format("LT", locale=locale) == "7:03"
    assert d.format("LTS", locale=locale) == "7:03:06"
    assert d.format("L", locale=locale) == "29. 8. 2016"
    assert d.format("LL", locale=locale) == "29. srpna, 2016"
    assert d.format("LLL", locale=locale) == "29. srpna, 2016 7:03"
    assert d.format("LLLL", locale=locale) == "pondělí, 29. srpna, 2016 7:03"
